@@ -27,12 +27,6 @@
 #include <stdlib.h>
 #include <locale.h>
 
-#ifdef HAVE_OSTREE
-#include <ostree.h>
-#include "asb-package-ostree.h"
-#endif
-
-#include "as-cleanup.h"
 #include "asb-context.h"
 #include "asb-utils.h"
 
@@ -43,13 +37,13 @@ static gboolean
 as_builder_search_path (GPtrArray *array, const gchar *path, GError **error)
 {
 	const gchar *filename;
-	_cleanup_dir_close_ GDir *dir = NULL;
+	g_autoptr(GDir) dir = NULL;
 
 	dir = g_dir_open (path, 0, error);
 	if (dir == NULL)
 		return FALSE;
 	while ((filename = g_dir_read_name (dir)) != NULL) {
-		_cleanup_free_ gchar *tmp = NULL;
+		g_autofree gchar *tmp = NULL;
 		tmp = g_build_filename (path, filename, NULL);
 		if (g_file_test (tmp, G_FILE_TEST_IS_DIR)) {
 			if (!as_builder_search_path (array, tmp, error))
@@ -61,63 +55,6 @@ as_builder_search_path (GPtrArray *array, const gchar *path, GError **error)
 	return TRUE;
 }
 
-/**
- * as_builder_setup_ostree:
- **/
-static gboolean
-as_builder_setup_ostree (AsbContext *ctx, const gchar *ostree_repo, GError **error)
-{
-#ifdef HAVE_OSTREE
-	GHashTableIter iter;
-	gpointer key;
-	gpointer value;
-	_cleanup_hashtable_unref_ GHashTable *refs = NULL;
-	_cleanup_object_unref_ GFile *file = NULL;
-	_cleanup_object_unref_ OstreeRepo *repo = NULL;
-
-	/* load repo */
-	file = g_file_new_for_path (ostree_repo);
-	repo = ostree_repo_new (file);
-	if (!ostree_repo_list_refs (repo, NULL, &refs, NULL, error))
-		return FALSE;
-
-	/* create a fake package for each of the refs */
-	g_hash_table_iter_init (&iter, refs);
-	while (g_hash_table_iter_next (&iter, &key, &value)) {
-		const gchar *refspec = key;
-		_cleanup_free_ gchar *remote = NULL;
-		_cleanup_free_ gchar *name = NULL;
-		_cleanup_free_ gchar *filename = NULL;
-		_cleanup_free_ gchar *ref = NULL;
-		_cleanup_object_unref_ AsbPackage *pkg = NULL;
-
-		if (!ostree_parse_refspec (refspec, &remote, &ref, error))
-			return FALSE;
-
-                if (!g_str_has_prefix (ref, "app/") &&
-                    !g_str_has_prefix (ref, "runtime/"))
-                  continue;
-
-		pkg = asb_package_ostree_new ();
-		asb_package_set_source (pkg, ref);
-		asb_package_ostree_set_repodir (ASB_PACKAGE_OSTREE (pkg), ostree_repo);
-
-		/* open the package */
-		filename = g_build_filename (ostree_repo, name, NULL);
-		if (!asb_package_open (pkg, filename, error))
-			return FALSE;
-		asb_context_add_package (ctx, pkg);
-	}
-	return TRUE;
-#else
-	g_set_error_literal (error,
-			     AS_UTILS_ERROR,
-			     AS_UTILS_ERROR_FAILED,
-			     "No ostree support enabled");
-	return FALSE;
-#endif
-}
-
 /**
  * main:
  **/
@@ -135,26 +72,24 @@ main (int argc, char **argv)
 	gboolean ret;
 	gboolean uncompressed_icons = FALSE;
 	gboolean verbose = FALSE;
-	gdouble api_version = 0.0f;
 	gint max_threads = 4;
 	gint min_icon_size = 32;
 	guint i;
 	int retval = EXIT_SUCCESS;
-	_cleanup_dir_close_ GDir *dir = NULL;
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_free_ gchar *basename = NULL;
-	_cleanup_free_ gchar *cache_dir = NULL;
-	_cleanup_free_ gchar *log_dir = NULL;
-	_cleanup_free_ gchar *icons_dir = NULL;
-	_cleanup_free_ gchar *old_metadata = NULL;
-	_cleanup_free_ gchar *origin = NULL;
-	_cleanup_free_ gchar *ostree_repo = NULL;
-	_cleanup_free_ gchar *output_dir = NULL;
-	_cleanup_free_ gchar *temp_dir = NULL;
-	_cleanup_free_ gchar **veto_ignore = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *packages = NULL;
-	_cleanup_strv_free_ gchar **packages_dirs = NULL;
-	_cleanup_timer_destroy_ GTimer *timer = NULL;
+	g_autoptr(GDir) dir = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autofree gchar *basename = NULL;
+	g_autofree gchar *cache_dir = NULL;
+	g_autofree gchar *log_dir = NULL;
+	g_autofree gchar *icons_dir = NULL;
+	g_autofree gchar *old_metadata = NULL;
+	g_autofree gchar *origin = NULL;
+	g_autofree gchar *output_dir = NULL;
+	g_autofree gchar *temp_dir = NULL;
+	g_autofree gchar **veto_ignore = NULL;
+	g_autoptr(GPtrArray) packages = NULL;
+	g_auto(GStrv) packages_dirs = NULL;
+	g_autoptr(GTimer) timer = NULL;
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
 			/* TRANSLATORS: command line option */
@@ -204,12 +139,6 @@ main (int argc, char **argv)
 		{ "min-icon-size", '\0', 0, G_OPTION_ARG_INT, &min_icon_size,
 			/* TRANSLATORS: command line option */
 			_("Set the minimum icon size in pixels"), "ICON_SIZE" },
-		{ "api-version", '\0', 0, G_OPTION_ARG_DOUBLE, &api_version,
-			/* TRANSLATORS: command line option */
-			_("Set the AppStream version"), "API_VERSION" },
-		{ "ostree-repo", '\0', 0, G_OPTION_ARG_STRING, &ostree_repo,
-			/* TRANSLATORS: command line option */
-			_("Set the ostree repo name"), "REPO" },
 		{ "old-metadata", '\0', 0, G_OPTION_ARG_FILENAME, &old_metadata,
 			/* TRANSLATORS: command line option */
 			_("Set the old metadata location"), "DIR" },
@@ -245,8 +174,6 @@ main (int argc, char **argv)
 	}
 #endif
 	/* set defaults */
-	if (api_version < 0.01)
-		api_version = 0.8;
 	if (temp_dir == NULL)
 		temp_dir = g_strdup ("./tmp");
 	if (log_dir == NULL)
@@ -259,14 +186,6 @@ main (int argc, char **argv)
 		cache_dir = g_strdup ("./cache");
 	setlocale (LC_ALL, "");
 
-	/* obsolete */
-	if (api_version < 0.6) {
-		/* TRANSLATORS: error message */
-		g_warning ("%s", _("API version no longer supported"));
-		retval = EXIT_FAILURE;
-		goto out;
-	}
-
 	/* this really ought to be set */
 	if (basename == NULL) {
 		g_print ("WARNING: Metadata basename not set, using 'appstream'\n");
@@ -278,7 +197,7 @@ main (int argc, char **argv)
 	}
 
 	ctx = asb_context_new ();
-	asb_context_set_api_version (ctx, api_version);
+	asb_context_set_api_version (ctx, 0.8);
 	asb_context_set_old_metadata (ctx, old_metadata);
 	asb_context_set_log_dir (ctx, log_dir);
 	asb_context_set_temp_dir (ctx, temp_dir);
@@ -350,10 +269,10 @@ main (int argc, char **argv)
 
 	/* scan each package */
 	packages = g_ptr_array_new_with_free_func (g_free);
-	if (argc == 1 && ostree_repo == NULL) {
+	if (argc == 1) {
 		/* if the user launches the tool with no arguments */
 		if (packages_dirs == NULL) {
-			_cleanup_free_ gchar *tmp = NULL;
+			g_autofree gchar *tmp = NULL;
 			tmp = g_option_context_get_help (option_context, TRUE, NULL);
 			g_print ("%s", tmp);
 			retval = EXIT_FAILURE;
@@ -375,7 +294,7 @@ main (int argc, char **argv)
 	g_print ("%s\n", _("Scanning packages..."));
 	timer = g_timer_new ();
 	for (i = 0; i < packages->len; i++) {
-		_cleanup_error_free_ GError *error_local = NULL;
+		g_autoptr(GError) error_local = NULL;
 
 		filename = g_ptr_array_index (packages, i);
 
@@ -394,15 +313,6 @@ main (int argc, char **argv)
 		}
 	}
 
-	/* load and data from an ostree repo */
-	if (ostree_repo != NULL) {
-		if (!as_builder_setup_ostree (ctx, ostree_repo, &error)) {
-			g_print ("%s\n", error->message);
-			retval = EXIT_FAILURE;
-			goto out;
-		}
-	}
-
 	/* process all packages in the pool */
 	ret = asb_context_process (ctx, &error);
 	if (!ret) {
