@@ -33,16 +33,14 @@
 #include <appstream-glib.h>
 
 #include "asb-app.h"
-#include "as-cleanup.h"
 
-typedef struct _AsbAppPrivate	AsbAppPrivate;
-struct _AsbAppPrivate
+typedef struct
 {
 	GPtrArray	*requires_appdata;
 	AsbPackage	*pkg;
 	gboolean	 ignore_requires_appdata;
 	gboolean	 hidpi_enabled;
-};
+} AsbAppPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbApp, asb_app, AS_TYPE_APP)
 
@@ -89,70 +87,6 @@ asb_app_class_init (AsbAppClass *klass)
 	object_class->finalize = asb_app_finalize;
 }
 
-/**
- * asb_app_add_requires_appdata:
- * @app: A #AsbApp
- * @fmt: format string
- * @...: varargs
- *
- * Adds a reason that AppData is required.
- *
- * Since: 0.1.0
- **/
-void
-asb_app_add_requires_appdata (AsbApp *app, const gchar *fmt, ...)
-{
-	AsbAppPrivate *priv = GET_PRIVATE (app);
-	gchar *tmp;
-	va_list args;
-	if (priv->ignore_requires_appdata)
-		return;
-	va_start (args, fmt);
-	tmp = g_strdup_vprintf (fmt, args);
-	va_end (args);
-	g_ptr_array_add (priv->requires_appdata, tmp);
-}
-
-/**
- * asb_app_set_requires_appdata:
- * @app: A #AsbApp
- * @requires_appdata: boolean
- *
- * Sets (or clears) the requirement for AppData.
- *
- * Since: 0.1.0
- **/
-void
-asb_app_set_requires_appdata (AsbApp *app, gboolean requires_appdata)
-{
-	AsbAppPrivate *priv = GET_PRIVATE (app);
-	if (requires_appdata) {
-		if (priv->ignore_requires_appdata)
-			return;
-		g_ptr_array_add (priv->requires_appdata, NULL);
-	} else {
-		g_ptr_array_set_size (priv->requires_appdata, 0);
-		priv->ignore_requires_appdata = TRUE;
-	}
-}
-
-/**
- * asb_app_get_requires_appdata:
- * @app: A #AsbApp
- *
- * Gets if AppData is still required for the application.
- *
- * Returns: (transfer none) (element-type utf8): A list of reasons
- *
- * Since: 0.1.0
- **/
-GPtrArray *
-asb_app_get_requires_appdata (AsbApp *app)
-{
-	AsbAppPrivate *priv = GET_PRIVATE (app);
-	return priv->requires_appdata;
-}
-
 /**
  * asb_app_get_package:
  * @app: A #AsbApp
@@ -170,6 +104,27 @@ asb_app_get_package (AsbApp *app)
 	return priv->pkg;
 }
 
+/**
+ * asb_app_set_package:
+ * @app: A #AsbApp
+ * @pkg: A #AsbPackage
+ *
+ * Sets the package that backs the application.
+ *
+ * Since: 0.5.1
+ **/
+void
+asb_app_set_package (AsbApp *app, AsbPackage *pkg)
+{
+	AsbAppPrivate *priv = GET_PRIVATE (app);
+
+	g_set_object (&priv->pkg, pkg);
+
+	/* be helpful */
+	if (asb_package_get_kind (pkg) == ASB_PACKAGE_KIND_DEFAULT)
+		as_app_add_pkgname (AS_APP (app), asb_package_get_name (pkg));
+}
+
 /**
  * asb_app_set_hidpi_enabled:
  * @app: A #AsbApp
@@ -212,12 +167,13 @@ asb_app_save_resources (AsbApp *app, AsbAppSaveFlags save_flags, GError **error)
 		icons = as_app_get_icons (AS_APP (app));
 	for (i = 0; icons != NULL && i < icons->len; i++) {
 		const gchar *tmpdir;
-		_cleanup_free_ gchar *filename = NULL;
-		_cleanup_free_ gchar *size_str = NULL;
+		g_autofree gchar *filename = NULL;
+		g_autofree gchar *size_str = NULL;
 
 		/* don't save some types of icons */
 		icon = g_ptr_array_index (icons, i);
-		if (as_icon_get_kind (icon) == AS_ICON_KIND_STOCK ||
+		if (as_icon_get_kind (icon) == AS_ICON_KIND_UNKNOWN ||
+		    as_icon_get_kind (icon) == AS_ICON_KIND_STOCK ||
 		    as_icon_get_kind (icon) == AS_ICON_KIND_EMBEDDED ||
 		    as_icon_get_kind (icon) == AS_ICON_KIND_LOCAL ||
 		    as_icon_get_kind (icon) == AS_ICON_KIND_REMOTE)
@@ -233,8 +189,9 @@ asb_app_save_resources (AsbApp *app, AsbAppSaveFlags save_flags, GError **error)
 			g_set_error (error,
 				     AS_APP_ERROR,
 				     AS_APP_ERROR_FAILED,
-				     "No pixbuf for %s",
-				     as_icon_get_name (icon));
+				     "No pixbuf for %s in %s",
+				     as_icon_get_name (icon),
+				     as_app_get_id (AS_APP (app)));
 			return FALSE;
 		}
 		if (!gdk_pixbuf_save (pixbuf, filename, "png", error, NULL))
@@ -250,8 +207,8 @@ asb_app_save_resources (AsbApp *app, AsbAppSaveFlags save_flags, GError **error)
 
 /**
  * asb_app_new:
- * @pkg: A #AsbPackage
- * @id: The ID for the package
+ * @pkg: A #AsbPackage, or %NULL
+ * @id: The ID for the package, or %NULL
  *
  * Creates a new application object.
  *
@@ -263,29 +220,9 @@ AsbApp *
 asb_app_new (AsbPackage *pkg, const gchar *id)
 {
 	AsbApp *app;
-	AsbAppPrivate *priv;
-
 	app = g_object_new (ASB_TYPE_APP, NULL);
-	priv = GET_PRIVATE (app);
-	if (pkg != NULL) {
-		priv->pkg = g_object_ref (pkg);
-		switch (asb_package_get_kind (pkg)) {
-		case ASB_PACKAGE_KIND_DEFAULT:
-			as_app_add_pkgname (AS_APP (app),
-					    asb_package_get_name (pkg));
-			break;
-		case ASB_PACKAGE_KIND_BUNDLE:
-		{
-			_cleanup_object_unref_ AsBundle *bundle = NULL;
-			bundle = as_bundle_new ();
-			as_bundle_set_id (bundle, asb_package_get_source (pkg));
-			as_bundle_set_kind (bundle, AS_BUNDLE_KIND_XDG_APP);
-			as_app_add_bundle (AS_APP (app), bundle);
-		};
-		default:
-			break;
-		}
-	}
+	if (pkg != NULL)
+		asb_app_set_package (app, pkg);
 	if (id != NULL)
 		as_app_set_id (AS_APP (app), id);
 	return ASB_APP (app);
