@@ -33,7 +33,6 @@
 #include <stdlib.h>
 #include <appstream-glib.h>
 
-#include "as-cleanup.h"
 #include "asb-context.h"
 #include "asb-context-private.h"
 #include "asb-plugin.h"
@@ -50,8 +49,7 @@
 #include "asb-package-cab.h"
 #include "asb-package-deb.h"
 
-typedef struct _AsbContextPrivate	AsbContextPrivate;
-struct _AsbContextPrivate
+typedef struct
 {
 	AsStore			*store_failed;
 	AsStore			*store_ignore;
@@ -73,7 +71,7 @@ struct _AsbContextPrivate
 	gchar			*icons_dir;
 	gchar			*basename;
 	gchar			*origin;
-};
+} AsbContextPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbContext, asb_context, G_TYPE_OBJECT)
 
@@ -467,7 +465,7 @@ asb_context_add_package (AsbContext *ctx, AsbPackage *pkg)
 gboolean
 asb_context_add_filename (AsbContext *ctx, const gchar *filename, GError **error)
 {
-	_cleanup_object_unref_ AsbPackage *pkg = NULL;
+	g_autoptr(AsbPackage) pkg = NULL;
 
 	/* can find in existing metadata */
 	if (asb_context_find_in_cache (ctx, filename)) {
@@ -542,9 +540,9 @@ gboolean
 asb_context_setup (AsbContext *ctx, GError **error)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	_cleanup_free_ gchar *icons_dir = NULL;
-	_cleanup_free_ gchar *screenshot_dir1 = NULL;
-	_cleanup_free_ gchar *screenshot_dir2 = NULL;
+	g_autofree gchar *icons_dir = NULL;
+	g_autofree gchar *screenshot_dir1 = NULL;
+	g_autofree gchar *screenshot_dir2 = NULL;
 
 	/* required stuff set */
 	if (priv->origin == NULL) {
@@ -601,8 +599,8 @@ asb_context_setup (AsbContext *ctx, GError **error)
 	if (!asb_utils_ensure_exists (priv->icons_dir, error))
 		return FALSE;
 	if (priv->flags & ASB_CONTEXT_FLAG_HIDPI_ICONS) {
-		_cleanup_free_ gchar *icons_dir_hidpi = NULL;
-		_cleanup_free_ gchar *icons_dir_lodpi = NULL;
+		g_autofree gchar *icons_dir_hidpi = NULL;
+		g_autofree gchar *icons_dir_lodpi = NULL;
 		icons_dir_lodpi = g_build_filename (priv->icons_dir, "64x64", NULL);
 		if (!asb_utils_ensure_exists (icons_dir_lodpi, error))
 			return FALSE;
@@ -613,7 +611,7 @@ asb_context_setup (AsbContext *ctx, GError **error)
 
 	/* decompress the icons */
 	if (priv->old_metadata != NULL) {
-		_cleanup_free_ gchar *icons_fn = NULL;
+		g_autofree gchar *icons_fn = NULL;
 		icons_fn = g_strdup_printf ("%s/%s-icons.tar.gz",
 					    priv->old_metadata,
 					    priv->basename);
@@ -635,13 +633,13 @@ asb_context_setup (AsbContext *ctx, GError **error)
 
 	/* add old metadata */
 	if (priv->old_metadata != NULL) {
-		_cleanup_free_ gchar *builder_id = NULL;
-		_cleanup_free_ gchar *fn_failed = NULL;
-		_cleanup_free_ gchar *fn_ignore = NULL;
-		_cleanup_free_ gchar *fn_old = NULL;
-		_cleanup_object_unref_ GFile *file_failed = NULL;
-		_cleanup_object_unref_ GFile *file_ignore = NULL;
-		_cleanup_object_unref_ GFile *file_old = NULL;
+		g_autofree gchar *builder_id = NULL;
+		g_autofree gchar *fn_failed = NULL;
+		g_autofree gchar *fn_ignore = NULL;
+		g_autofree gchar *fn_old = NULL;
+		g_autoptr(GFile) file_failed = NULL;
+		g_autoptr(GFile) file_ignore = NULL;
+		g_autoptr(GFile) file_old = NULL;
 
 		builder_id = asb_utils_get_builder_id ();
 		fn_old = g_strdup_printf ("%s/%s.xml.gz",
@@ -707,7 +705,7 @@ static void
 asb_task_process_func (gpointer data, gpointer user_data)
 {
 	AsbTask *task = (AsbTask *) data;
-	_cleanup_error_free_ GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	/* just run the task */
 	if (!asb_task_process (task, &error))
@@ -723,7 +721,7 @@ asb_context_write_icons (AsbContext *ctx,
 			 GError **error)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	_cleanup_free_ gchar *filename = NULL;
+	g_autofree gchar *filename = NULL;
 
 	/* not enabled */
 	if (priv->flags & ASB_CONTEXT_FLAG_UNCOMPRESSED_ICONS)
@@ -746,8 +744,8 @@ asb_context_write_screenshots (AsbContext *ctx,
 			       GError **error)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_free_ gchar *screenshot_dir = NULL;
+	g_autofree gchar *filename = NULL;
+	g_autofree gchar *screenshot_dir = NULL;
 
 	/* not enabled */
 	if (priv->flags & ASB_CONTEXT_FLAG_UNCOMPRESSED_ICONS)
@@ -771,9 +769,9 @@ asb_context_write_xml (AsbContext *ctx, GError **error)
 	AsApp *app;
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	GList *l;
-	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_object_unref_ AsStore *store = NULL;
-	_cleanup_object_unref_ GFile *file = NULL;
+	g_autofree gchar *filename = NULL;
+	g_autoptr(AsStore) store = NULL;
+	g_autoptr(GFile) file = NULL;
 
 	/* convert any vetod applications into dummy components */
 	for (l = priv->apps; l != NULL; l = l->next) {
@@ -797,7 +795,7 @@ asb_context_write_xml (AsbContext *ctx, GError **error)
 		/* remove from the ignore list if the application was useful */
 		if (ASB_IS_APP (app)) {
 			AsbPackage *pkg = asb_app_get_package (ASB_APP (app));
-			_cleanup_free_ gchar *name_arch = NULL;
+			g_autofree gchar *name_arch = NULL;
 			name_arch = g_strdup_printf ("%s.%s",
 						     asb_package_get_name (pkg),
 						     asb_package_get_arch (pkg));
@@ -814,7 +812,7 @@ asb_context_write_xml (AsbContext *ctx, GError **error)
 	as_store_set_origin (store, priv->origin);
 	as_store_set_api_version (store, priv->api_version);
 	if (priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) {
-		_cleanup_free_ gchar *builder_id = asb_utils_get_builder_id ();
+		g_autofree gchar *builder_id = asb_utils_get_builder_id ();
 		as_store_set_builder_id (store, builder_id);
 	}
 	return as_store_to_file (store,
@@ -887,7 +885,7 @@ asb_context_detect_pkgname_dups (AsbContext *ctx, GError **error)
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	GList *l;
 	const gchar *pkgname;
-	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
+	g_autoptr(GHashTable) hash = NULL;
 
 	hash = g_hash_table_new (g_str_hash, g_str_equal);
 	for (l = priv->apps; l != NULL; l = l->next) {
@@ -921,8 +919,8 @@ asb_context_write_app_xml (AsbContext *ctx)
 
 	/* log the XML in the log file */
 	for (l = priv->apps; l != NULL; l = l->next) {
-		_cleanup_string_free_ GString *xml = NULL;
-		_cleanup_object_unref_ AsStore *store = NULL;
+		g_autoptr(GString) xml = NULL;
+		g_autoptr(AsStore) store = NULL;
 
 		/* we have an open log file? */
 		if (!ASB_IS_APP (l->data))
@@ -955,12 +953,18 @@ asb_context_detect_missing_data (AsbContext *ctx, GError **error)
 	for (l = priv->apps; l != NULL; l = l->next) {
 		app = AS_APP (l->data);
 		if (as_app_get_name (AS_APP (app), "C") == NULL)
-			as_app_add_veto (AS_APP (app), "No 'Name' in desktop or <name> in AppData");
+			as_app_add_veto (AS_APP (app), "No <name> in AppData");
 		if (as_app_get_comment (AS_APP (app), "C") == NULL)
-			as_app_add_veto (AS_APP (app), "No 'Comment' in desktop or <summary> in AppData");
-		if (as_app_get_id_kind (AS_APP (app)) != AS_ID_KIND_ADDON) {
+			as_app_add_veto (AS_APP (app), "No <summary> in AppData");
+		switch (as_app_get_kind (AS_APP (app))) {
+		case AS_APP_KIND_ADDON:
+		case AS_APP_KIND_FIRMWARE:
+		case AS_APP_KIND_GENERIC:
+			break;
+		default:
 			if (as_app_get_icon_default (AS_APP (app)) == NULL)
 				as_app_add_veto (AS_APP (app), "Has no Icon");
+			break;
 		}
 	}
 	return TRUE;
@@ -977,7 +981,7 @@ asb_context_detect_missing_parents (AsbContext *ctx, GError **error)
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	GList *l;
 	const gchar *tmp;
-	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
+	g_autoptr(GHashTable) hash = NULL;
 
 	/* add all desktop apps to the hash */
 	hash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -987,7 +991,7 @@ asb_context_detect_missing_parents (AsbContext *ctx, GError **error)
 			continue;
 		if (as_app_get_pkgname_default (app) == NULL)
 			continue;
-		if (as_app_get_id_kind (app) != AS_ID_KIND_DESKTOP)
+		if (as_app_get_kind (app) != AS_APP_KIND_DESKTOP)
 			continue;
 		g_hash_table_insert (hash,
 				     (gpointer) as_app_get_id (app),
@@ -1001,7 +1005,7 @@ asb_context_detect_missing_parents (AsbContext *ctx, GError **error)
 			continue;
 		if (as_app_get_pkgname_default (app) == NULL)
 			continue;
-		if (as_app_get_id_kind (app) != AS_ID_KIND_ADDON)
+		if (as_app_get_kind (app) != AS_APP_KIND_ADDON)
 			continue;
 		if (as_app_get_extends(app)->len == 0)
 			continue;
@@ -1014,8 +1018,8 @@ asb_context_detect_missing_parents (AsbContext *ctx, GError **error)
 			continue;
 
 		/* do not add the addon */
-		as_app_add_veto (app, "%s has no parent of '%s'\n",
-				  as_app_get_id (app), tmp);
+		as_app_add_veto (app, "%s has no parent of '%s'",
+				 as_app_get_id (app), tmp);
 		g_print ("WARNING: %s has no parent of '%s'\n",
 			 as_app_get_id (app), tmp);
 	}
@@ -1031,9 +1035,9 @@ asb_context_write_xml_fail (AsbContext *ctx, GError **error)
 	AsApp *app;
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	GList *l;
-	_cleanup_free_ gchar *basename_failed = NULL;
-	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_object_unref_ GFile *file = NULL;
+	g_autofree gchar *basename_failed = NULL;
+	g_autofree gchar *filename = NULL;
+	g_autoptr(GFile) file = NULL;
 
 	/* no need to create */
 	if ((priv->flags & ASB_CONTEXT_FLAG_INCLUDE_FAILED) == 0)
@@ -1045,8 +1049,6 @@ asb_context_write_xml_fail (AsbContext *ctx, GError **error)
 			continue;
 		if (as_app_get_vetos(app)->len == 0)
 			continue;
-		if (as_app_get_metadata_item (app, "NoDisplay") != NULL)
-			continue;
 		if (as_store_get_app_by_id (priv->store_failed,
 					    as_app_get_id (app)) != NULL)
 			continue;
@@ -1061,7 +1063,7 @@ asb_context_write_xml_fail (AsbContext *ctx, GError **error)
 	as_store_set_origin (priv->store_failed, basename_failed);
 	as_store_set_api_version (priv->store_failed, priv->api_version);
 	if (priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) {
-		_cleanup_free_ gchar *builder_id = asb_utils_get_builder_id ();
+		g_autofree gchar *builder_id = asb_utils_get_builder_id ();
 		as_store_set_builder_id (priv->store_failed, builder_id);
 	}
 	return as_store_to_file (priv->store_failed,
@@ -1079,9 +1081,9 @@ static gboolean
 asb_context_write_xml_ignore (AsbContext *ctx, GError **error)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	_cleanup_free_ gchar *basename_cache = NULL;
-	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_object_unref_ GFile *file = NULL;
+	g_autofree gchar *basename_cache = NULL;
+	g_autofree gchar *filename = NULL;
+	g_autoptr(GFile) file = NULL;
 
 	/* no need to create */
 	if ((priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) == 0)
@@ -1097,7 +1099,7 @@ asb_context_write_xml_ignore (AsbContext *ctx, GError **error)
 	as_store_set_origin (priv->store_ignore, basename_cache);
 	as_store_set_api_version (priv->store_ignore, priv->api_version);
 	if (priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) {
-		_cleanup_free_ gchar *builder_id = asb_utils_get_builder_id ();
+		g_autofree gchar *builder_id = asb_utils_get_builder_id ();
 		as_store_set_builder_id (priv->store_ignore, builder_id);
 	}
 	return as_store_to_file (priv->store_ignore,
@@ -1119,7 +1121,7 @@ asb_context_disable_older_pkgs (AsbContext *ctx)
 	AsbPackage *pkg;
 	const gchar *key;
 	guint i;
-	_cleanup_hashtable_unref_ GHashTable *newest = NULL;
+	g_autoptr(GHashTable) newest = NULL;
 
 	newest = g_hash_table_new_full (g_str_hash, g_str_equal,
 					g_free, (GDestroyNotify) g_object_unref);
@@ -1208,7 +1210,7 @@ asb_context_process (AsbContext *ctx, GError **error)
 	GThreadPool *pool;
 	gboolean ret;
 	guint i;
-	_cleanup_ptrarray_unref_ GPtrArray *tasks = NULL;
+	g_autoptr(GPtrArray) tasks = NULL;
 
 	/* only process the newest packages */
 	asb_context_disable_multiarch_pkgs (ctx);
@@ -1341,10 +1343,10 @@ asb_context_find_in_cache (AsbContext *ctx, const gchar *filename)
 	AsApp *app;
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	guint i;
-	_cleanup_free_ gchar *cache_id = NULL;
-	_cleanup_free_ gchar *builder_id = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *apps = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *apps_ignore = NULL;
+	g_autofree gchar *cache_id = NULL;
+	g_autofree gchar *builder_id = NULL;
+	g_autoptr(GPtrArray) apps = NULL;
+	g_autoptr(GPtrArray) apps_ignore = NULL;
 
 	/* the package was successfully parsed last time */
 	cache_id = asb_utils_get_cache_id_for_filename (filename);
@@ -1421,9 +1423,9 @@ asb_context_add_app_ignore (AsbContext *ctx, AsbPackage *pkg)
 {
 	AsApp *app_tmp;
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	_cleanup_free_ gchar *name_arch = NULL;
-	_cleanup_object_unref_ AsApp *app = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *apps = NULL;
+	g_autofree gchar *name_arch = NULL;
+	g_autoptr(AsApp) app = NULL;
+	g_autoptr(GPtrArray) apps = NULL;
 
 	/* only do this when we are using a cache-id */
 	if ((priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) == 0)
