@@ -33,16 +33,14 @@
 
 #include <alpm.h>
 
-#include "as-cleanup.h"
 #include "asb-package-alpm.h"
 #include "asb-plugin.h"
 
-typedef struct _AsbPackageAlpmPrivate	AsbPackageAlpmPrivate;
-struct _AsbPackageAlpmPrivate
+typedef struct
 {
 	alpm_handle_t	*handle;
 	alpm_pkg_t	*package;
-};
+} AsbPackageAlpmPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbPackageAlpm, asb_package_alpm, ASB_TYPE_PACKAGE)
 
@@ -123,7 +121,7 @@ asb_package_alpm_ensure_version (AsbPackage *pkg, GError **error)
 	AsbPackageAlpm *pkg_alpm = ASB_PACKAGE_ALPM (pkg);
 	AsbPackageAlpmPrivate *priv = GET_PRIVATE (pkg_alpm);
 
-	gchar _cleanup_strv_free_ **split = NULL;
+	g_auto(GStrv) split = NULL;
 
 	split = g_strsplit (alpm_pkg_get_version (priv->package), ":-", 3);
 
