@@ -30,17 +30,15 @@
 
 #include "config.h"
 
-#include "as-cleanup.h"
 #include "asb-plugin-loader.h"
 #include "asb-plugin.h"
 
-typedef struct _AsbPluginLoaderPrivate	AsbPluginLoaderPrivate;
-struct _AsbPluginLoaderPrivate
+typedef struct
 {
 	GPtrArray		*plugins;
 	AsbContext		*ctx;
 	gchar			*plugin_dir;
-};
+} AsbPluginLoaderPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbPluginLoader, asb_plugin_loader, G_TYPE_OBJECT)
 
@@ -188,18 +186,11 @@ asb_plugin_loader_process_app (AsbPluginLoader *plugin_loader,
 				 "Running asb_plugin_process_app() from %s",
 				 plugin->name);
 		if (!plugin_func (plugin, pkg, app, tmpdir, &error_local)) {
-			if (g_error_matches (error_local,
-					     ASB_PLUGIN_ERROR,
-					     ASB_PLUGIN_ERROR_IGNORE)) {
-				asb_package_log (pkg,
-						 ASB_PACKAGE_LOG_LEVEL_WARNING,
-						 "Ignoring: %s",
-						 error_local->message);
-				g_clear_error (&error_local);
-			} else {
-				g_propagate_error (error, error_local);
-				return FALSE;
-			}
+			asb_package_log (pkg,
+					 ASB_PACKAGE_LOG_LEVEL_WARNING,
+					 "Ignoring: %s",
+					 error_local->message);
+			g_clear_error (&error_local);
 		}
 	}
 	return TRUE;
@@ -278,7 +269,7 @@ asb_plugin_loader_merge (AsbPluginLoader *plugin_loader, GList *apps)
 	const gchar *tmp;
 	gboolean ret;
 	guint i;
-	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
+	g_autoptr(GHashTable) hash = NULL;
 
 	/* run each plugin */
 	for (i = 0; i < priv->plugins->len; i++) {
@@ -321,7 +312,7 @@ asb_plugin_loader_merge (AsbPluginLoader *plugin_loader, GList *apps)
 					     (gpointer) app);
 			continue;
 		}
-		if (as_app_get_id_kind (AS_APP (app)) == AS_ID_KIND_FIRMWARE) {
+		if (as_app_get_kind (AS_APP (app)) == AS_APP_KIND_FIRMWARE) {
 			as_app_subsume_full (AS_APP (found), AS_APP (app),
 					     AS_APP_SUBSUME_FLAG_PARTIAL);
 		}
@@ -438,7 +429,7 @@ asb_plugin_loader_setup (AsbPluginLoader *plugin_loader, GError **error)
 {
 	AsbPluginLoaderPrivate *priv = GET_PRIVATE (plugin_loader);
 	const gchar *filename_tmp;
-	_cleanup_dir_close_ GDir *dir = NULL;
+	g_autoptr(GDir) dir = NULL;
 
 	/* never set */
 	if (priv->plugin_dir == NULL)
@@ -452,7 +443,7 @@ asb_plugin_loader_setup (AsbPluginLoader *plugin_loader, GError **error)
 	/* try to open each plugin */
 	g_debug ("searching for plugins in %s", priv->plugin_dir);
 	do {
-		_cleanup_free_ gchar *filename_plugin = NULL;
+		g_autofree gchar *filename_plugin = NULL;
 		filename_tmp = g_dir_read_name (dir);
 		if (filename_tmp == NULL)
 			break;
