@@ -24,8 +24,7 @@
 #include <glib.h>
 #include <stdlib.h>
 #include <locale.h>
-
-#include "as-cleanup.h"
+#include <fnmatch.h>
 
 #include "asb-context-private.h"
 #include "asb-plugin.h"
@@ -43,9 +42,7 @@
 static gchar *
 asb_test_get_filename (const gchar *filename)
 {
-	char full_tmp[PATH_MAX];
-	gchar *tmp;
-	_cleanup_free_ gchar *path = NULL;
+	g_autofree gchar *path = NULL;
 
 	/* try the source then the destdir */
 	path = g_build_filename (TESTDIRSRC, filename, NULL);
@@ -53,24 +50,28 @@ asb_test_get_filename (const gchar *filename)
 		g_free (path);
 		path = g_build_filename (TESTDIRBUILD, filename, NULL);
 	}
-	tmp = realpath (path, full_tmp);
-	if (tmp == NULL)
-		return NULL;
-	return g_strdup (full_tmp);
+	return realpath (path, NULL);
 }
 
+#define AS_TEST_WILDCARD_SHA1	"\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?"
+#define AS_TEST_WILDCARD_MD5	"\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?\?"
+
 /**
  * asb_test_compare_lines:
  **/
 static gboolean
 asb_test_compare_lines (const gchar *txt1, const gchar *txt2, GError **error)
 {
-	_cleanup_free_ gchar *output = NULL;
+	g_autofree gchar *output = NULL;
 
 	/* exactly the same */
 	if (g_strcmp0 (txt1, txt2) == 0)
 		return TRUE;
 
+	/* matches a pattern */
+	if (fnmatch (txt2, txt1, FNM_NOESCAPE) == 0)
+		return TRUE;
+
 	/* save temp files and diff them */
 	if (!g_file_set_contents ("/tmp/a", txt1, -1, error))
 		return FALSE;
@@ -95,9 +96,9 @@ asb_test_package_rpm_func (void)
 	GPtrArray *releases;
 	gboolean ret;
 	gchar *tmp;
-	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_object_unref_ AsbPackage *pkg = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *glob = NULL;
+	g_autofree gchar *filename = NULL;
+	g_autoptr(AsbPackage) pkg = NULL;
+	g_autoptr(GPtrArray) glob = NULL;
 
 	/* open file */
 	filename = asb_test_get_filename ("test-0.1-1.fc21.noarch.rpm");
@@ -211,19 +212,43 @@ asb_test_package_rpm_func (void)
 static void
 asb_test_package_func (void)
 {
-	_cleanup_object_unref_ AsbPackage *pkg = NULL;
+	g_autoptr(AsbPackage) pkg = NULL;
+	g_autoptr(AsbPackage) pkg2 = NULL;
+
+	/* set package values from filename */
 	pkg = asb_package_new ();
 	asb_package_set_filename (pkg, "/tmp/gambit-c-doc-4.7.3-2.fc22.noarch.rpm");
 	g_assert_cmpstr (asb_package_get_nevra (pkg), ==, "gambit-c-doc-4.7.3-2.fc22.noarch");
 	g_assert_cmpstr (asb_package_get_name (pkg), ==, "gambit-c-doc");
 	g_assert_cmpstr (asb_package_get_version (pkg), ==, "4.7.3");
+	g_assert_cmpstr (asb_package_get_release_str (pkg), ==, "2.fc22");
 	g_assert_cmpstr (asb_package_get_arch (pkg), ==, "noarch");
+	g_assert_cmpint (asb_package_get_epoch (pkg), ==, 0);
+
+	/* set package values again */
+	pkg2 = asb_package_new ();
+	asb_package_set_filename (pkg2, "/tmp/gambit-c-doc-4.7.3-2.fc22.noarch.rpm");
+
+	/* check same */
+	g_assert_cmpint (asb_package_compare (pkg, pkg2), ==, 0);
+
+	/* fix version */
+	asb_package_set_version (pkg2, "4.7.4");
+	g_assert_cmpint (asb_package_compare (pkg, pkg2), <, 0);
+	g_assert_cmpint (asb_package_compare (pkg2, pkg), >, 0);
+	asb_package_set_version (pkg2, "4.7.3");
+
+	/* fix release */
+	asb_package_set_release (pkg2, "3.fc22");
+	g_assert_cmpint (asb_package_compare (pkg, pkg2), <, 0);
+	g_assert_cmpint (asb_package_compare (pkg2, pkg), >, 0);
+	asb_package_set_release (pkg2, "2.fc22");
 }
 
 static void
 asb_test_utils_glob_func (void)
 {
-	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
+	g_autoptr(GPtrArray) array = NULL;
 
 	array = asb_glob_value_array_new ();
 	g_ptr_array_add (array, asb_glob_value_new ("*.desktop", "DESKTOP"));
@@ -234,18 +259,6 @@ asb_test_utils_glob_func (void)
 	g_assert_cmpstr (asb_glob_value_search (array, "gimp.appdata.xml"), ==, "APPDATA");
 }
 
-static void
-asb_test_utils_replace_func (void)
-{
-	guint n;
-	_cleanup_string_free_ GString *str = NULL;
-
-	str = g_string_new ("I want to have searched for search");
-	n = asb_string_replace (str, "search", "replace");
-	g_assert_cmpstr (str->str, ==, "I want to have replaceed for replace");
-	g_assert_cmpint (n, ==, 2);
-}
-
 static void
 asb_test_plugin_loader_func (void)
 {
@@ -254,8 +267,8 @@ asb_test_plugin_loader_func (void)
 	GError *error = NULL;
 	GPtrArray *plugins;
 	gboolean ret;
-	_cleanup_object_unref_ AsbContext *ctx = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *globs = NULL;
+	g_autoptr(AsbContext) ctx = NULL;
+	g_autoptr(GPtrArray) globs = NULL;
 
 	/* set up loader */
 	ctx = asb_context_new ();
@@ -267,14 +280,13 @@ asb_test_plugin_loader_func (void)
 
 	/* get the list of globs */
 	globs = asb_plugin_loader_get_globs (loader);\
-	g_assert_cmpint (globs->len, >=, 44);
+	g_assert_cmpint (globs->len, >=, 10);
 	g_assert_cmpstr (asb_glob_value_search (globs, "/usr/share/applications/gimp.desktop"), ==, "");
-	g_assert_cmpstr (asb_glob_value_search (globs, "/files/share/applications/gimp.desktop"), ==, "");
 	g_assert_cmpstr (asb_glob_value_search (globs, "/srv/dave.txt"), ==, NULL);
 
 	/* get the list of plugins */
 	plugins = asb_plugin_loader_get_plugins (loader);
-	g_assert_cmpint (plugins->len, >=, 17);
+	g_assert_cmpint (plugins->len, >=, 5);
 	plugin = g_ptr_array_index (plugins, 0);
 	g_assert (plugin != NULL);
 	g_assert (plugin->module != NULL);
@@ -282,9 +294,9 @@ asb_test_plugin_loader_func (void)
 	g_assert (plugin->ctx == ctx);
 
 	/* match the correct one */
-	plugin = asb_plugin_loader_match_fn (loader, "/usr/share/applications/gimp.desktop");
+	plugin = asb_plugin_loader_match_fn (loader, "/usr/share/appdata/gimp.appdata.xml");
 	g_assert (plugin != NULL);
-	g_assert_cmpstr (plugin->name, ==, "desktop");
+	g_assert_cmpstr (plugin->name, ==, "appdata");
 }
 
 #ifdef HAVE_RPM
@@ -305,16 +317,16 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	const gchar *expected_xml;
 	gboolean ret;
 	guint i;
-	_cleanup_object_unref_ AsbContext *ctx = NULL;
-	_cleanup_object_unref_ AsStore *store_failed = NULL;
-	_cleanup_object_unref_ AsStore *store_ignore = NULL;
-	_cleanup_object_unref_ AsStore *store = NULL;
-	_cleanup_object_unref_ GFile *file_failed = NULL;
-	_cleanup_object_unref_ GFile *file_ignore = NULL;
-	_cleanup_object_unref_ GFile *file = NULL;
-	_cleanup_string_free_ GString *xml = NULL;
-	_cleanup_string_free_ GString *xml_failed = NULL;
-	_cleanup_string_free_ GString *xml_ignore = NULL;
+	g_autoptr(AsbContext) ctx = NULL;
+	g_autoptr(AsStore) store_failed = NULL;
+	g_autoptr(AsStore) store_ignore = NULL;
+	g_autoptr(AsStore) store = NULL;
+	g_autoptr(GFile) file_failed = NULL;
+	g_autoptr(GFile) file_ignore = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GString) xml = NULL;
+	g_autoptr(GString) xml_failed = NULL;
+	g_autoptr(GString) xml_ignore = NULL;
 	const gchar *filenames[] = {
 		"test-0.1-1.fc21.noarch.rpm",		/* a console app */
 		"app-1-1.fc21.x86_64.rpm",		/* a GUI app */
@@ -348,7 +360,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		break;
 	case ASB_TEST_CONTEXT_MODE_WITH_OLD_CACHE:
 		{
-			_cleanup_free_ gchar *old_cache_dir = NULL;
+			g_autofree gchar *old_cache_dir = NULL;
 			old_cache_dir = asb_test_get_filename (".");
 			asb_context_set_old_metadata (ctx, old_cache_dir);
 		}
@@ -366,7 +378,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 
 	/* add packages */
 	for (i = 0; filenames[i] != NULL; i++) {
-		_cleanup_free_ gchar *filename = NULL;
+		g_autofree gchar *filename = NULL;
 		filename = asb_test_get_filename (filenames[i]);
 		if (filename == NULL)
 			g_warning ("%s not found", filenames[i]);
@@ -406,7 +418,11 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	ret = as_store_from_file (store, file, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
-	g_assert_cmpint (as_store_get_size (store), ==, 6);
+#ifdef HAVE_FONTS
+	g_assert_cmpint (as_store_get_size (store), ==, 4);
+#else
+	g_assert_cmpint (as_store_get_size (store), ==, 3);
+#endif
 	app = as_store_get_app_by_pkgname (store, "app");
 	g_assert (app != NULL);
 	app = as_store_get_app_by_id (store, "app.desktop");
@@ -416,6 +432,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	xml = as_store_to_xml (store, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
 		"<components version=\"0.9\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test\">\n"
+#ifdef HAVE_FONTS
 		"<component type=\"font\">\n"
 		"<id>Liberation</id>\n"
 		"<pkgname>font</pkgname>\n"
@@ -430,15 +447,15 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<category>Fonts</category>\n"
 		"</categories>\n"
 		"<project_license>GPL-2.0+</project_license>\n"
-		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
+		"<url type=\"homepage\">http://fedorahosted.org/liberation-fonts/</url>\n"
 		"<screenshots>\n"
 		"<screenshot type=\"default\">\n"
 		"<caption>Liberation Serif – Regular</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-85c191549daefb8f3b6cdb4d3d6d8597.png</image>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-" AS_TEST_WILDCARD_MD5 ".png</image>\n"
 		"</screenshot>\n"
 		"<screenshot priority=\"-32\">\n"
 		"<caption>Liberation Serif – Bold</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-660e7a5a60fdeb133d3e8b24e2766f2e.png</image>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-" AS_TEST_WILDCARD_MD5 ".png</image>\n"
 		"</screenshot>\n"
 		"</screenshots>\n"
 		"<releases>\n"
@@ -451,6 +468,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<value key=\"X-CacheID\">font-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
+#endif
 		"<component type=\"addon\">\n"
 		"<id>app-extra</id>\n"
 		"<pkgname>app-extra</pkgname>\n"
@@ -459,6 +477,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<summary>Addons for extra functionality</summary>\n"
 		"<kudos>\n"
 		"<kudo>ModernToolkit</kudo>\n"
+		"<kudo>SearchProvider</kudo>\n"
 		"</kudos>\n"
 		"<project_license>GPL-2.0+</project_license>\n"
 		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
@@ -466,6 +485,9 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<releases>\n"
 		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
 		"</releases>\n"
+		"<provides>\n"
+		"<dbus type=\"session\">org.freedesktop.AppStream</dbus>\n"
+		"</provides>\n"
 		"<languages>\n"
 		"<lang percentage=\"100\">en_GB</lang>\n"
 		"<lang percentage=\"33\">ru</lang>\n"
@@ -490,13 +512,11 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<keyword>Remote</keyword>\n"
 		"</keywords>\n"
 		"<kudos>\n"
-		"<kudo>AppMenu</kudo>\n"
 		"<kudo>ModernToolkit</kudo>\n"
-		"<kudo>Notifications</kudo>\n"
 		"<kudo>SearchProvider</kudo>\n"
 		"<kudo>UserDocs</kudo>\n"
 		"</kudos>\n"
-		"<project_license>GPL-2.0+</project_license>\n"
+		"<project_license>LGPL-2.0+</project_license>\n"
 		"<url type=\"homepage\">http://people.freedesktop.org/~hughsient/appdata/</url>\n"
 		"<screenshots>\n"
 		"<screenshot type=\"default\">\n"
@@ -517,6 +537,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<value key=\"X-CacheID\">app-1-1.fc21.x86_64.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
+#ifdef HAVE_GCAB
 		"<component type=\"firmware\">\n"
 		"<id>com.hughski.ColorHug2.firmware</id>\n"
 		"<name>ColorHug Firmware</name>\n"
@@ -524,17 +545,19 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<developer_name>Hughski Limited</developer_name>\n"
 		"<description><p>Updating the firmware on your ColorHug device "
 		"improves performance and adds new features.</p></description>\n"
-		"<icon type=\"stock\">application-x-executable</icon>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
 		"<url type=\"homepage\">http://www.hughski.com/</url>\n"
 		"<releases>\n"
 		"<release version=\"2.0.2\" timestamp=\"1424116753\">\n"
 		"<location>http://www.hughski.com/downloads/colorhug2/firmware/colorhug-2.0.2.cab</location>\n"
-		"<checksum filename=\"colorhug-als-2.0.2.cab\" target=\"container\" type=\"sha1\">6301e24ae5fb2aa73c336fd40a5d2f1835afe088</checksum>\n"
-		"<checksum filename=\"firmware.bin\" target=\"content\" type=\"sha1\">767a8a7b8a7b350b513f57761204b4aaa657aa44</checksum>\n"
+		"<checksum filename=\"colorhug-als-2.0.2.cab\" target=\"container\" type=\"sha1\">" AS_TEST_WILDCARD_SHA1 "</checksum>\n"
+		"<checksum filename=\"firmware.bin\" target=\"content\" type=\"sha1\">" AS_TEST_WILDCARD_SHA1 "</checksum>\n"
 		"<description><p>This unstable release adds the following features:</p>"
 		"<ul><li>Add TakeReadingArray to enable panel latency measurements</li>"
 		"<li>Speed up the auto-scaled measurements considerably, using 256ms "
 		"as the smallest sample duration</li></ul></description>\n"
+		"<size type=\"installed\">14</size>\n"
+		"<size type=\"download\">2011</size>\n"
 		"</release>\n"
 		"</releases>\n"
 		"<provides>\n"
@@ -544,48 +567,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<value key=\"X-CacheID\">colorhug-als-2.0.2.cab</value>\n"
 		"</metadata>\n"
 		"</component>\n"
-		"<component type=\"desktop\">\n"
-		"<id>valid1.desktop</id>\n"
-		"<pkgname>composite</pkgname>\n"
-		"<name>Frobnicator</name>\n"
-		"<summary>Frobnicator</summary>\n"
-		"<icon type=\"stock\">computer</icon>\n"
-		"<categories>\n"
-		"<category>Profiling</category>\n"
-		"</categories>\n"
-		"<kudos>\n"
-		"<kudo>HiDpiIcon</kudo>\n"
-		"</kudos>\n"
-		"<project_license>GPL-2.0+</project_license>\n"
-		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
-		"<releases>\n"
-		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
-		"</releases>\n"
-		"<metadata>\n"
-		"<value key=\"X-CacheID\">composite-1-1.fc21.x86_64.rpm</value>\n"
-		"</metadata>\n"
-		"</component>\n"
-		"<component type=\"desktop\">\n"
-		"<id>valid2.desktop</id>\n"
-		"<pkgname>composite</pkgname>\n"
-		"<name>Frobnicator Example</name>\n"
-		"<summary>Frobnicator Example Program</summary>\n"
-		"<icon type=\"stock\">computer</icon>\n"
-		"<categories>\n"
-		"<category>Profiling</category>\n"
-		"</categories>\n"
-		"<kudos>\n"
-		"<kudo>HiDpiIcon</kudo>\n"
-		"</kudos>\n"
-		"<project_license>GPL-2.0+</project_license>\n"
-		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
-		"<releases>\n"
-		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
-		"</releases>\n"
-		"<metadata>\n"
-		"<value key=\"X-CacheID\">composite-1-1.fc21.x86_64.rpm</value>\n"
-		"</metadata>\n"
-		"</component>\n"
+#endif
 		"</components>\n";
 	ret = asb_test_compare_lines (xml->str, expected_xml, &error);
 	g_assert_no_error (error);
@@ -597,16 +579,17 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	ret = as_store_from_file (store_failed, file_failed, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
-	g_assert_cmpint (as_store_get_size (store_failed), ==, 4);
-	app = as_store_get_app_by_id (store_failed, "console1.desktop");
-	g_assert (app != NULL);
-	app = as_store_get_app_by_id (store_failed, "console2.desktop");
-	g_assert (app != NULL);
+	g_assert_cmpint (as_store_get_size (store_failed), ==, 2);
+//	app = as_store_get_app_by_id (store_failed, "console1.desktop");
+//	g_assert (app != NULL);
+//	app = as_store_get_app_by_id (store_failed, "console2.desktop");
+//	g_assert (app != NULL);
 
 	/* check output */
 	xml_failed = as_store_to_xml (store_failed, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
 		"<components version=\"0.9\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test-failed\">\n"
+#ifdef HAVE_FONTS
 		"<component type=\"font\">\n"
 		"<id>LiberationSerif</id>\n"
 		"<pkgname>font-serif</pkgname>\n"
@@ -627,11 +610,11 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<screenshots>\n"
 		"<screenshot type=\"default\">\n"
 		"<caption>Liberation Serif – Regular</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-85c191549daefb8f3b6cdb4d3d6d8597.png</image>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-" AS_TEST_WILDCARD_MD5 ".png</image>\n"
 		"</screenshot>\n"
 		"<screenshot priority=\"-32\">\n"
 		"<caption>Liberation Serif – Bold</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-660e7a5a60fdeb133d3e8b24e2766f2e.png</image>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-" AS_TEST_WILDCARD_MD5 ".png</image>\n"
 		"</screenshot>\n"
 		"</screenshots>\n"
 		"<releases>\n"
@@ -644,15 +627,14 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<value key=\"X-CacheID\">font-serif-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
+#endif
 		"<component type=\"addon\">\n"
 		"<id>app-core</id>\n"
 		"<pkgname>app</pkgname>\n"
 		"<name>Core</name>\n"
 		"<summary>Addons for core functionality</summary>\n"
 		"<kudos>\n"
-		"<kudo>AppMenu</kudo>\n"
 		"<kudo>ModernToolkit</kudo>\n"
-		"<kudo>Notifications</kudo>\n"
 		"<kudo>SearchProvider</kudo>\n"
 		"<kudo>UserDocs</kudo>\n"
 		"</kudos>\n"
@@ -677,64 +659,6 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<value key=\"X-Merge-With-Parent\">app.desktop</value>\n"
 		"</metadata>\n"
 		"</component>\n"
-		"<component type=\"desktop\">\n"
-		"<id>console1.desktop</id>\n"
-		"<pkgname>app-console</pkgname>\n"
-		"<source_pkgname>app</source_pkgname>\n"
-		"<name>Console1</name>\n"
-		"<summary>A console1 test application</summary>\n"
-		"<icon height=\"64\" width=\"64\" type=\"cached\">console1.png</icon>\n"
-		"<categories>\n"
-		"<category>ConsoleOnly</category>\n"
-		"</categories>\n"
-		"<kudos>\n"
-		"<kudo>ModernToolkit</kudo>\n"
-		"</kudos>\n"
-		"<vetos>\n"
-		"<veto>Required AppData: ConsoleOnly</veto>\n"
-		"</vetos>\n"
-		"<project_license>GPL-2.0+</project_license>\n"
-		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
-		"<releases>\n"
-		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
-		"</releases>\n"
-		"<languages>\n"
-		"<lang percentage=\"100\">en_GB</lang>\n"
-		"<lang percentage=\"33\">ru</lang>\n"
-		"</languages>\n"
-		"<metadata>\n"
-		"<value key=\"X-CacheID\">app-console-1-1.fc21.noarch.rpm</value>\n"
-		"</metadata>\n"
-		"</component>\n"
-		"<component type=\"desktop\">\n"
-		"<id>console2.desktop</id>\n"
-		"<pkgname>app-console</pkgname>\n"
-		"<source_pkgname>app</source_pkgname>\n"
-		"<name>Console2</name>\n"
-		"<summary>A console2 test application</summary>\n"
-		"<icon height=\"64\" width=\"64\" type=\"cached\">console2.png</icon>\n"
-		"<categories>\n"
-		"<category>ConsoleOnly</category>\n"
-		"</categories>\n"
-		"<kudos>\n"
-		"<kudo>ModernToolkit</kudo>\n"
-		"</kudos>\n"
-		"<vetos>\n"
-		"<veto>Required AppData: ConsoleOnly</veto>\n"
-		"</vetos>\n"
-		"<project_license>GPL-2.0+</project_license>\n"
-		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
-		"<releases>\n"
-		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
-		"</releases>\n"
-		"<languages>\n"
-		"<lang percentage=\"100\">en_GB</lang>\n"
-		"<lang percentage=\"33\">ru</lang>\n"
-		"</languages>\n"
-		"<metadata>\n"
-		"<value key=\"X-CacheID\">app-console-1-1.fc21.noarch.rpm</value>\n"
-		"</metadata>\n"
-		"</component>\n"
 		"</components>\n";
 	ret = asb_test_compare_lines (xml_failed->str, expected_xml, &error);
 	g_assert_no_error (error);
@@ -751,28 +675,35 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	xml_ignore = as_store_to_xml (store_ignore, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
 		"<components version=\"0.9\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test-ignore\">\n"
-		"<component>\n"
+		"<component type=\"generic\">\n"
 		"<id>app-console.noarch</id>\n"
 		"<pkgname>app-console</pkgname>\n"
 		"<metadata>\n"
 		"<value key=\"X-CacheID\">app-console-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
-		"<component>\n"
+		"<component type=\"generic\">\n"
 		"<id>app.i686</id>\n"
 		"<pkgname>app</pkgname>\n"
 		"<metadata>\n"
 		"<value key=\"X-CacheID\">app-1-1.fc21.i686.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
-		"<component>\n"
+		"<component type=\"generic\">\n"
+		"<id>composite.x86_64</id>\n"
+		"<pkgname>composite</pkgname>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">composite-1-1.fc21.x86_64.rpm</value>\n"
+		"</metadata>\n"
+		"</component>\n"
+		"<component type=\"generic\">\n"
 		"<id>font-serif.noarch</id>\n"
 		"<pkgname>font-serif</pkgname>\n"
 		"<metadata>\n"
 		"<value key=\"X-CacheID\">font-serif-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
-		"<component>\n"
+		"<component type=\"generic\">\n"
 		"<id>test.noarch</id>\n"
 		"<pkgname>test</pkgname>\n"
 		"<metadata>\n"
@@ -852,16 +783,17 @@ asb_test_context_oldcache_func (void)
 static void
 asb_test_firmware_func (void)
 {
+#ifdef HAVE_GCAB
 	AsApp *app;
 	AsbPluginLoader *loader;
 	const gchar *expected_xml;
 	gboolean ret;
 	guint i;
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_object_unref_ AsbContext *ctx = NULL;
-	_cleanup_object_unref_ AsStore *store = NULL;
-	_cleanup_object_unref_ GFile *file = NULL;
-	_cleanup_string_free_ GString *xml = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(AsbContext) ctx = NULL;
+	g_autoptr(AsStore) store = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GString) xml = NULL;
 	const gchar *filenames[] = {
 		"colorhug-als-2.0.1.cab",
 		"colorhug-als-2.0.0.cab",
@@ -887,7 +819,7 @@ asb_test_firmware_func (void)
 
 	/* add packages */
 	for (i = 0; filenames[i] != NULL; i++) {
-		_cleanup_free_ gchar *filename = NULL;
+		g_autofree gchar *filename = NULL;
 		filename = asb_test_get_filename (filenames[i]);
 		if (filename == NULL)
 			g_warning ("%s not found", filenames[i]);
@@ -929,30 +861,30 @@ asb_test_firmware_func (void)
 		"<developer_name>Hughski Limited</developer_name>\n"
 		"<description><p>Updating the firmware on your ColorHug device "
 		"improves performance and adds new features.</p></description>\n"
-		"<icon type=\"stock\">application-x-executable</icon>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
 		"<url type=\"homepage\">http://www.hughski.com/</url>\n"
 		"<releases>\n"
 		"<release version=\"2.0.2\" timestamp=\"1424116753\">\n"
 		"<location>http://www.hughski.com/downloads/colorhug2/firmware/colorhug-2.0.2.cab</location>\n"
-		"<checksum filename=\"colorhug-als-2.0.2.cab\" target=\"container\" type=\"sha1\">6301e24ae5fb2aa73c336fd40a5d2f1835afe088</checksum>\n"
-		"<checksum filename=\"firmware.bin\" target=\"content\" type=\"sha1\">767a8a7b8a7b350b513f57761204b4aaa657aa44</checksum>\n"
+		"<checksum filename=\"colorhug-als-2.0.2.cab\" target=\"container\" type=\"sha1\">" AS_TEST_WILDCARD_SHA1 "</checksum>\n"
+		"<checksum filename=\"firmware.bin\" target=\"content\" type=\"sha1\">" AS_TEST_WILDCARD_SHA1 "</checksum>\n"
 		"<description><p>This unstable release adds the following features:</p>"
 		"<ul><li>Add TakeReadingArray to enable panel latency measurements</li>"
 		"<li>Speed up the auto-scaled measurements considerably, using 256ms "
 		"as the smallest sample duration</li></ul></description>\n"
+		"<size type=\"installed\">14</size>\n"
+		"<size type=\"download\">2011</size>\n"
 		"</release>\n"
 		"<release version=\"2.0.1\" timestamp=\"1424116753\">\n"
 		"<location>http://www.hughski.com/downloads/colorhug2/firmware/colorhug-2.0.1.cab</location>\n"
-		"<checksum filename=\"colorhug-als-2.0.1.cab\" target=\"container\" type=\"sha1\">7d4cab8775213cbd223a4a3b236618ced0af9447</checksum>\n"
-		"<checksum filename=\"firmware.bin\" target=\"content\" type=\"sha1\">767a8a7b8a7b350b513f57761204b4aaa657aa44</checksum>\n"
+		"<checksum filename=\"colorhug-als-2.0.1.cab\" target=\"container\" type=\"sha1\">" AS_TEST_WILDCARD_SHA1 "</checksum>\n"
+		"<checksum filename=\"firmware.bin\" target=\"content\" type=\"sha1\">" AS_TEST_WILDCARD_SHA1 "</checksum>\n"
 		"<description><p>This unstable release adds the following features:</p>"
 		"<ul><li>Use TakeReadings() to do a quick non-adaptive measurement</li>"
 		"<li>Scale XYZ measurement with a constant factor to make the CCMX more "
 		"sane</li></ul></description>\n"
-		"</release>\n"
-		"<release version=\"2.0.0\" timestamp=\"1425168000\">\n"
-		"<checksum filename=\"colorhug-als-2.0.0.cab\" target=\"container\" type=\"sha1\">d7b9ae5b36029de606c9b047c229d9c8f5e3807c</checksum>\n"
-		"<checksum filename=\"firmware.bin\" target=\"content\" type=\"sha1\">767a8a7b8a7b350b513f57761204b4aaa657aa44</checksum>\n"
+		"<size type=\"installed\">14</size>\n"
+		"<size type=\"download\">1951</size>\n"
 		"</release>\n"
 		"</releases>\n"
 		"<provides>\n"
@@ -967,6 +899,7 @@ asb_test_firmware_func (void)
 	ret = asb_utils_rmtree ("/tmp/asbuilder", &error);
 	g_assert_no_error (error);
 	g_assert (ret);
+#endif
 }
 
 int
@@ -981,7 +914,6 @@ main (int argc, char **argv)
 
 	/* tests go here */
 	g_test_add_func ("/AppStreamBuilder/package", asb_test_package_func);
-	g_test_add_func ("/AppStreamBuilder/utils{replace}", asb_test_utils_replace_func);
 	g_test_add_func ("/AppStreamBuilder/utils{glob}", asb_test_utils_glob_func);
 	g_test_add_func ("/AppStreamBuilder/plugin-loader", asb_test_plugin_loader_func);
 	g_test_add_func ("/AppStreamBuilder/firmware", asb_test_firmware_func);
