@@ -30,7 +30,6 @@
 
 #include "config.h"
 
-#include "as-cleanup.h"
 #include "asb-context-private.h"
 #include "asb-task.h"
 #include "asb-package.h"
@@ -38,15 +37,14 @@
 #include "asb-plugin.h"
 #include "asb-plugin-loader.h"
 
-typedef struct _AsbTaskPrivate	AsbTaskPrivate;
-struct _AsbTaskPrivate
+typedef struct
 {
 	AsbContext		*ctx;
 	AsbPackage		*pkg;
 	GPtrArray		*plugins_to_run;
 	gchar			*filename;
 	gchar			*tmpdir;
-};
+} AsbTaskPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbTask, asb_task, G_TYPE_OBJECT)
 
@@ -159,9 +157,9 @@ asb_task_explode_extra_packages (AsbTask *task, GError **error)
 	const gchar *ignore[] = { "rtld", NULL };
 	const gchar *tmp;
 	guint i;
-	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
-	_cleanup_ptrarray_unref_ GPtrArray *icon_themes = NULL;
+	g_autoptr(GHashTable) hash = NULL;
+	g_autoptr(GPtrArray) array = NULL;
+	g_autoptr(GPtrArray) icon_themes = NULL;
 
 	/* anything the package requires */
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
@@ -238,11 +236,10 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 	GPtrArray *array;
 	gboolean ret;
 	gchar *cache_id;
-	gchar *tmp;
 	guint i;
 	guint nr_added = 0;
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_free_ gchar *basename = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autofree gchar *basename = NULL;
 
 	/* reset the profile timer */
 	asb_package_log_start (priv->pkg);
@@ -255,6 +252,42 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 
 	g_debug ("starting: %s", asb_package_get_name (priv->pkg));
 
+	/* treat archive as a special case */
+	if (g_str_has_suffix (priv->filename, ".cab")) {
+		AsApp *app_tmp;
+		GPtrArray *apps_tmp;
+		g_autoptr(AsStore) store = as_store_new ();
+		g_autoptr(GFile) file = g_file_new_for_path (priv->filename);
+		if (!as_store_from_file (store, file, NULL, NULL, &error)) {
+			asb_package_log (priv->pkg,
+					 ASB_PACKAGE_LOG_LEVEL_WARNING,
+					 "Failed to parse %s: %s",
+					 asb_package_get_filename (priv->pkg),
+					 error->message);
+			return TRUE;
+		}
+		apps_tmp = as_store_get_apps (store);
+		for (i = 0; i < apps_tmp->len; i++) {
+			g_autoptr(AsbApp) app2 = NULL;
+			app_tmp = AS_APP (g_ptr_array_index (apps_tmp, i));
+			app2 = asb_app_new (priv->pkg, as_app_get_id (app_tmp));
+			as_app_subsume (AS_APP (app2), app_tmp);
+			asb_context_add_app (priv->ctx, app2);
+
+			/* set cache-id in case we want to use the metadata directly */
+			if (asb_context_get_flag (priv->ctx, ASB_CONTEXT_FLAG_ADD_CACHE_ID)) {
+				cache_id = asb_utils_get_cache_id_for_filename (priv->filename);
+				as_app_add_metadata (AS_APP (app2),
+						     "X-CacheID",
+						     cache_id);
+				g_free (cache_id);
+			}
+			nr_added++;
+		}
+		g_debug ("added %i apps from archive", apps_tmp->len);
+		goto skip;
+	}
+
 	/* ensure file list read */
 	if (!asb_package_ensure (priv->pkg,
 				 ASB_PACKAGE_ENSURE_FILES,
@@ -364,12 +397,14 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 					 ASB_PACKAGE_ENSURE_URL,
 					 error_not_used))
 			return FALSE;
-		if (asb_package_get_url (priv->pkg) != NULL) {
+		if (asb_package_get_url (priv->pkg) != NULL &&
+		    as_app_get_url_item (AS_APP (app), AS_URL_KIND_HOMEPAGE) == NULL) {
 			as_app_add_url (AS_APP (app),
 					AS_URL_KIND_HOMEPAGE,
 					asb_package_get_url (priv->pkg));
 		}
-		if (asb_package_get_license (priv->pkg) != NULL) {
+		if (asb_package_get_license (priv->pkg) != NULL &&
+		    as_app_get_project_license (AS_APP (app)) == NULL) {
 			as_app_set_project_license (AS_APP (app),
 						    asb_package_get_license (priv->pkg));
 		}
@@ -404,17 +439,6 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 			goto skip;
 		}
 
-		/* veto apps that *still* require appdata */
-		array = asb_app_get_requires_appdata (app);
-		for (i = 0; i < array->len; i++) {
-			tmp = g_ptr_array_index (array, i);
-			if (tmp == NULL) {
-				as_app_add_veto (AS_APP (app), "Required AppData");
-				continue;
-			}
-			as_app_add_veto (AS_APP (app), "Required AppData: %s", tmp);
-		}
-
 		/* set cache-id in case we want to use the metadata directly */
 		if (asb_context_get_flag (priv->ctx, ASB_CONTEXT_FLAG_ADD_CACHE_ID)) {
 			cache_id = asb_utils_get_cache_id_for_filename (priv->filename);
