@@ -25,16 +25,24 @@
 #define __APPSTREAM_GLIB_H_INSIDE__
 
 #include <as-app.h>
+#include <as-app-builder.h>
+#include <as-bundle.h>
+#include <as-checksum.h>
+#include <as-content-rating.h>
 #include <as-enums.h>
 #include <as-icon.h>
 #include <as-image.h>
+#include <as-inf.h>
+#include <as-markup.h>
 #include <as-node.h>
 #include <as-problem.h>
+#include <as-profile.h>
 #include <as-provide.h>
 #include <as-release.h>
 #include <as-screenshot.h>
 #include <as-store.h>
 #include <as-tag.h>
+#include <as-translation.h>
 #include <as-version.h>
 #include <as-utils.h>
 
