@@ -23,7 +23,6 @@
 #include <string.h>
 
 #include "as-app-private.h"
-#include "as-cleanup.h"
 #include "as-utils.h"
 
 /**
@@ -56,7 +55,7 @@ as_app_infer_file_key (AsApp *app,
 		       const gchar *key,
 		       GError **error)
 {
-	_cleanup_free_ gchar *tmp = NULL;
+	g_autofree gchar *tmp = NULL;
 
 	if (g_strcmp0 (key, "X-GNOME-UsesNotifications") == 0) {
 		as_app_add_kudo_kind (AS_APP (app),
@@ -131,9 +130,9 @@ as_app_parse_file_key (AsApp *app,
 	gchar *dot = NULL;
 	guint i;
 	guint j;
-	_cleanup_free_ gchar *locale = NULL;
-	_cleanup_free_ gchar *tmp = NULL;
-	_cleanup_strv_free_ gchar **list = NULL;
+	g_autofree gchar *locale = NULL;
+	g_autofree gchar *tmp = NULL;
+	g_auto(GStrv) list = NULL;
 
 	/* NoDisplay */
 	if (g_strcmp0 (key, G_KEY_FILE_DESKTOP_KEY_NO_DISPLAY) == 0) {
@@ -165,12 +164,23 @@ as_app_parse_file_key (AsApp *app,
 					     key,
 					     NULL);
 		if (tmp != NULL && tmp[0] != '\0') {
-			_cleanup_object_unref_ AsIcon *icon = NULL;
+			g_autoptr(AsIcon) icon = NULL;
 			icon = as_icon_new ();
+
+			if (g_path_is_absolute (tmp)) {
+				as_icon_set_filename (icon, tmp);
+			} else {
+				/* work around a common mistake in desktop files */
+				dot = g_strstr_len (tmp, -1, ".");
+				if (dot != NULL &&
+				    (g_strcmp0 (dot, ".png") == 0 ||
+				     g_strcmp0 (dot, ".xpm") == 0 ||
+				     g_strcmp0 (dot, ".svg") == 0)) {
+					*dot = '\0';
+				}
+			}
 			as_icon_set_name (icon, tmp);
-			dot = g_strstr_len (tmp, -1, ".");
-			if (dot != NULL)
-				*dot = '\0';
+
 			if (as_utils_is_stock_icon_name (tmp)) {
 				as_icon_set_name (icon, tmp);
 				as_icon_set_kind (icon, AS_ICON_KIND_STOCK);
@@ -191,14 +201,17 @@ as_app_parse_file_key (AsApp *app,
 						   key,
 						   NULL, NULL);
 		for (i = 0; list[i] != NULL; i++) {
+			const gchar *category_blacklist[] = {
+				"X-GNOME-Settings-Panel",
+				"X-Unity-Settings-Panel",
+				NULL };
 
-			/* check categories that if present would blacklist
-			 * the application */
-			if (fnmatch ("X-*-Settings-Panel", list[i], 0) == 0 ||
-			    fnmatch ("X-*-Settings", list[i], 0) == 0 ||
-			    fnmatch ("X-*-SettingsDialog", list[i], 0) == 0) {
-				as_app_add_veto (app, "category '%s' blacklisted", list[i]);
-				continue;
+			/* we have to veto these */
+			for (j = 0; category_blacklist[j] != NULL; j++) {
+				if (g_strcmp0 (list[i], category_blacklist[j]) == 0) {
+					as_app_add_veto (app, "Has category %s",
+							 category_blacklist[j]);
+				}
 			}
 
 			/* not a standard category */
@@ -227,7 +240,7 @@ as_app_parse_file_key (AsApp *app,
 						   key,
 						   NULL, NULL);
 		for (i = 0; list[i] != NULL; i++) {
-			_cleanup_strv_free_ gchar **kw_split = NULL;
+			g_auto(GStrv) kw_split = NULL;
 			kw_split = g_strsplit (list[i], ",", -1);
 			for (j = 0; kw_split[j] != NULL; j++) {
 				if (kw_split[j][0] == '\0')
@@ -244,7 +257,7 @@ as_app_parse_file_key (AsApp *app,
 							  locale,
 							  NULL, NULL);
 		for (i = 0; list[i] != NULL; i++) {
-			_cleanup_strv_free_ gchar **kw_split = NULL;
+			g_auto(GStrv) kw_split = NULL;
 			kw_split = g_strsplit (list[i], ",", -1);
 			for (j = 0; kw_split[j] != NULL; j++) {
 				if (kw_split[j][0] == '\0')
@@ -352,8 +365,8 @@ as_app_parse_file_key_fallback_comment (AsApp *app,
 					const gchar *key,
 					GError **error)
 {
-	_cleanup_free_ gchar *locale = NULL;
-	_cleanup_free_ gchar *tmp = NULL;
+	g_autofree gchar *locale = NULL;
+	g_autofree gchar *tmp = NULL;
 
 	/* GenericName */
 	if (g_strcmp0 (key, G_KEY_FILE_DESKTOP_KEY_GENERIC_NAME) == 0 ||
@@ -392,10 +405,10 @@ as_app_parse_desktop_file (AsApp *app,
 	GKeyFileFlags kf_flags = G_KEY_FILE_KEEP_TRANSLATIONS;
 	gchar *tmp;
 	guint i;
-	_cleanup_error_free_ GError *error_local = NULL;
-	_cleanup_free_ gchar *app_id = NULL;
-	_cleanup_keyfile_unref_ GKeyFile *kf = NULL;
-	_cleanup_strv_free_ gchar **keys = NULL;
+	g_autoptr(GError) error_local = NULL;
+	g_autofree gchar *app_id = NULL;
+	g_autoptr(GKeyFile) kf = NULL;
+	g_auto(GStrv) keys = NULL;
 
 	/* load file */
 	kf = g_key_file_new ();
@@ -410,13 +423,32 @@ as_app_parse_desktop_file (AsApp *app,
 		return FALSE;
 	}
 
+	/* check this is a valid desktop file */
+	if (!g_key_file_has_group (kf, G_KEY_FILE_DESKTOP_GROUP)) {
+		g_set_error (error,
+			     AS_APP_ERROR,
+			     AS_APP_ERROR_INVALID_TYPE,
+			     "Not a desktop file: no [%s]",
+			     G_KEY_FILE_DESKTOP_GROUP);
+		return FALSE;
+	}
+
 	/* create app */
 	app_id = g_path_get_basename (desktop_file);
-	as_app_set_id_kind (app, AS_ID_KIND_DESKTOP);
+	as_app_set_kind (app, AS_APP_KIND_DESKTOP);
 
-	/* is blacklisted */
-	if (as_utils_is_blacklisted_id (app_id))
-		as_app_add_veto (app, "%s is not an application", app_id);
+	/* is this really a web-app? */
+	if ((flags & AS_APP_PARSE_FLAG_USE_HEURISTICS) > 0) {
+		g_autofree gchar *exec = NULL;
+		exec = g_key_file_get_string (kf,
+					      G_KEY_FILE_DESKTOP_GROUP,
+					      G_KEY_FILE_DESKTOP_KEY_EXEC,
+					      NULL);
+		if (exec != NULL) {
+			if (g_str_has_prefix (exec, "epiphany"))
+				as_app_set_kind (app, AS_APP_KIND_WEB_APP);
+		}
+	}
 
 	/* Ubuntu helpfully put the package name in the desktop file name */
 	tmp = g_strstr_len (app_id, -1, ":");
