@@ -23,8 +23,8 @@
 #include <string.h>
 
 #include "as-app-private.h"
-#include "as-cleanup.h"
 #include "as-inf.h"
+#include "as-utils-private.h"
 
 #define AS_APP_INF_CLASS_GUID_FIRMWARE	"f2e7dd72-6468-4e36-b6f1-6488f42c1b52"
 
@@ -56,27 +56,30 @@ as_app_parse_inf_file (AsApp *app,
 		       GError **error)
 {
 	guint64 timestamp;
-	_cleanup_error_free_ GError *error_local = NULL;
-	_cleanup_free_ gchar *appstream_id = NULL;
-	_cleanup_free_ gchar *catalog_basename = NULL;
-	_cleanup_free_ gchar *class_guid = NULL;
-	_cleanup_free_ gchar *class_guid_unsafe = NULL;
-	_cleanup_free_ gchar *class = NULL;
-	_cleanup_free_ gchar *comment = NULL;
-	_cleanup_free_ gchar *filename_full = NULL;
-	_cleanup_free_ gchar *firmware_basename = NULL;
-	_cleanup_free_ gchar *guid = NULL;
-	_cleanup_free_ gchar *provide_guid = NULL;
-	_cleanup_free_ gchar *location_checksum = NULL;
-	_cleanup_free_ gchar *name = NULL;
-	_cleanup_free_ gchar *srcpkg = NULL;
-	_cleanup_free_ gchar *vendor = NULL;
-	_cleanup_free_ gchar *version = NULL;
-	_cleanup_keyfile_unref_ GKeyFile *kf = NULL;
-	_cleanup_object_unref_ AsIcon *icon = NULL;
-	_cleanup_object_unref_ AsProvide *provide = NULL;
-	_cleanup_object_unref_ AsRelease *release = NULL;
-	_cleanup_strv_free_ gchar **source_keys = NULL;
+	g_autoptr(GError) error_local = NULL;
+	g_autofree gchar *appstream_id = NULL;
+	g_autofree gchar *catalog_basename = NULL;
+	g_autofree gchar *class_guid = NULL;
+	g_autofree gchar *class_guid_unsafe = NULL;
+	g_autofree gchar *class = NULL;
+	g_autofree gchar *comment = NULL;
+	g_autofree gchar *display_version = NULL;
+	g_autofree gchar *filename_full = NULL;
+	g_autofree gchar *firmware_basename = NULL;
+	g_autofree gchar *guid = NULL;
+	g_autofree gchar *provide_guid = NULL;
+	g_autofree gchar *location_checksum = NULL;
+	g_autofree gchar *name = NULL;
+	g_autofree gchar *srcpkg = NULL;
+	g_autofree gchar *vendor = NULL;
+	g_autofree gchar *version = NULL;
+	g_autofree gchar *version_parsed = NULL;
+	g_autoptr(GKeyFile) kf = NULL;
+	g_autoptr(AsChecksum) csum = NULL;
+	g_autoptr(AsIcon) icon = NULL;
+	g_autoptr(AsProvide) provide = NULL;
+	g_autoptr(AsRelease) release = NULL;
+	g_auto(GStrv) source_keys = NULL;
 
 	/* load file */
 	kf = g_key_file_new ();
@@ -104,8 +107,8 @@ as_app_parse_inf_file (AsApp *app,
 			     AS_APP_ERROR_INVALID_TYPE,
 			     "Driver class is '%s', not 'Firmware'", class);
 		return FALSE;
-       }
-	as_app_set_id_kind (app, AS_ID_KIND_FIRMWARE);
+	}
+	as_app_set_kind (app, AS_APP_KIND_FIRMWARE);
 
 	/* get the Class GUID */
 	class_guid_unsafe = g_key_file_get_string (kf, "Version", "ClassGuid", NULL);
@@ -140,6 +143,22 @@ as_app_parse_inf_file (AsApp *app,
 		return FALSE;
 	}
 
+	/* get the version */
+	version = g_key_file_get_string (kf,
+					"Firmware_AddReg",
+					"HKR_FirmwareVersion_0x00010001",
+					NULL);
+	if (version == NULL) {
+		g_set_error_literal (error,
+				     AS_APP_ERROR,
+				     AS_APP_ERROR_INVALID_TYPE,
+				     "HKR->FirmwareVersion missing from [Firmware_AddReg]");
+		return FALSE;
+	}
+
+	/* convert if required */
+	version_parsed = as_utils_version_parse (version);
+
 	/* add the GUID as a provide */
 	provide_guid = as_app_parse_inf_sanitize_guid (guid);
 	if (provide_guid != NULL) {
@@ -178,8 +197,8 @@ as_app_parse_inf_file (AsApp *app,
 		as_app_set_comment (app, NULL, comment);
 
 	/* parse the DriverVer */
-	version = as_inf_get_driver_version (kf, &timestamp, &error_local);
-	if (version == NULL) {
+	display_version = as_inf_get_driver_version (kf, &timestamp, &error_local);
+	if (display_version == NULL) {
 		if (g_error_matches (error_local,
 				     AS_INF_ERROR,
 				     AS_INF_ERROR_NOT_FOUND)) {
@@ -209,20 +228,20 @@ as_app_parse_inf_file (AsApp *app,
 			     "no SourceDisksFiles or Firmware_CopyFiles");
 		return FALSE;
 	}
-	as_app_add_metadata (app, "FirmwareBasename", firmware_basename);
 
-	/* optional */
-	catalog_basename = g_key_file_get_string (kf, "Version", "CatalogFile", NULL);
-	if (catalog_basename != NULL)
-		as_app_add_metadata (app, "CatalogBasename", catalog_basename);
+	/* this is for display only */
+	if (display_version != NULL)
+		as_app_add_metadata (app, "DisplayVersion", display_version);
 
 	/* add a release with no real description */
-	if (version != NULL) {
-		release = as_release_new ();
-		as_release_set_version (release, version);
-		as_release_set_timestamp (release, timestamp);
-		as_app_add_release (app, release);
-	}
+	release = as_release_new ();
+	as_release_set_version (release, version_parsed);
+	as_release_set_timestamp (release, timestamp);
+	csum = as_checksum_new ();
+	as_checksum_set_filename (csum, firmware_basename);
+	as_checksum_set_target (csum, AS_CHECKSUM_TARGET_CONTENT);
+	as_release_add_checksum (release, csum);
+	as_app_add_release (app, release);
 
 	/* add icon */
 	icon = as_icon_new ();
