@@ -48,6 +48,9 @@ G_BEGIN_DECLS
  * @AS_APP_PROBLEM_TRANSLATED_ID:		The <id> value was translated
  * @AS_APP_PROBLEM_TRANSLATED_LICENSE:		The <license> value was translated
  * @AS_APP_PROBLEM_TRANSLATED_PROJECT_GROUP:	The <project_group> value was translated
+ * @AS_APP_PROBLEM_UPDATECONTACT_FALLBACK:	The file used <updatecontact> without a space
+ * @AS_APP_PROBLEM_INVALID_PROJECT_GROUP:	Invalid project group detected
+ * @AS_APP_PROBLEM_INVALID_XML_TAG:		Invalid XML tag name detected
  *
  * The application problems detected when loading.
  **/
@@ -65,6 +68,9 @@ typedef enum {
 	AS_APP_PROBLEM_TRANSLATED_ID		= 1 << 9,
 	AS_APP_PROBLEM_TRANSLATED_LICENSE	= 1 << 10,
 	AS_APP_PROBLEM_TRANSLATED_PROJECT_GROUP	= 1 << 11,
+	AS_APP_PROBLEM_UPDATECONTACT_FALLBACK	= 1 << 12,
+	AS_APP_PROBLEM_INVALID_PROJECT_GROUP	= 1 << 13,
+	AS_APP_PROBLEM_INVALID_XML_TAG		= 1 << 14,
 	/*< private >*/
 	AS_APP_PROBLEM_LAST
 } AsAppProblems;
