@@ -27,7 +27,6 @@
 #include <string.h>
 
 #include "as-app-private.h"
-#include "as-cleanup.h"
 #include "as-node-private.h"
 #include "as-problem.h"
 #include "as-utils.h"
@@ -54,7 +53,7 @@ ai_app_validate_add (AsAppValidateHelper *helper,
 	AsProblem *problem;
 	guint i;
 	va_list args;
-	_cleanup_free_ gchar *str = NULL;
+	g_autofree gchar *str = NULL;
 
 	va_start (args, fmt);
 	str = g_strdup_vprintf (fmt, args);
@@ -139,7 +138,7 @@ as_app_validate_has_email (const gchar *text)
 static gboolean
 as_app_validate_has_first_word_capital (AsAppValidateHelper *helper, const gchar *text)
 {
-	_cleanup_free_ gchar *first_word = NULL;
+	g_autofree gchar *first_word = NULL;
 	gchar *tmp;
 	guint i;
 
@@ -187,6 +186,14 @@ as_app_validate_description_li (const gchar *text, AsAppValidateHelper *helper)
 		require_sentence_case = FALSE;
 	}
 
+	/* empty */
+	if (text == NULL) {
+		ai_app_validate_add (helper,
+				     AS_PROBLEM_KIND_STYLE_INCORRECT,
+				     "<li> is empty");
+		return;
+	}
+
 	str_len = strlen (text);
 	if (str_len < length_li_min) {
 		ai_app_validate_add (helper,
@@ -341,7 +348,7 @@ as_app_validate_description (const gchar *xml,
 {
 	GNode *l;
 	GNode *l2;
-	_cleanup_node_unref_ GNode *node = NULL;
+	g_autoptr(AsNode) node = NULL;
 
 	/* parse xml */
 	node = as_node_from_xml (xml, AS_NODE_FROM_XML_FLAG_NONE, error);
@@ -443,10 +450,10 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	guint ss_size_height_min = 351;
 	guint ss_size_width_max = 1600;
 	guint ss_size_width_min = 624;
-	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
-	_cleanup_object_unref_ GInputStream *stream = NULL;
-	_cleanup_object_unref_ SoupMessage *msg = NULL;
-	_cleanup_uri_unref_ SoupURI *base_uri = NULL;
+	g_autoptr(GdkPixbuf) pixbuf = NULL;
+	g_autoptr(GInputStream) stream = NULL;
+	g_autoptr(SoupMessage) msg = NULL;
+	g_autoptr(SoupURI) base_uri = NULL;
 
 	/* make the requirements more strict */
 	if ((helper->flags & AS_APP_VALIDATE_FLAG_STRICT) > 0) {
@@ -469,7 +476,7 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	url = as_image_get_url (im);
 	g_debug ("checking %s", url);
 	base_uri = soup_uri_new (url);
-	if (base_uri == NULL) {
+	if (!SOUP_URI_VALID_FOR_HTTP (base_uri)) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_URL_NOT_FOUND,
 				     "<screenshot> url not valid [%s]", url);
@@ -717,9 +724,9 @@ as_app_validate_icons (AsApp *app, AsAppValidateHelper *helper)
 
 	/* check the content is correct */
 	icon_kind = as_icon_get_kind (icon);
-	icon_name = as_icon_get_name (icon);
 	switch (icon_kind) {
 	case AS_ICON_KIND_STOCK:
+		icon_name = as_icon_get_name (icon);
 		if (!as_utils_is_stock_icon_name (icon_name)) {
 			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
@@ -728,6 +735,7 @@ as_app_validate_icons (AsApp *app, AsAppValidateHelper *helper)
 		}
 		break;
 	case AS_ICON_KIND_LOCAL:
+		icon_name = as_icon_get_filename (icon);
 		if (!g_str_has_prefix (icon_name, "/")) {
 			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
@@ -736,6 +744,7 @@ as_app_validate_icons (AsApp *app, AsAppValidateHelper *helper)
 		}
 		break;
 	case AS_ICON_KIND_CACHED:
+		icon_name = as_icon_get_name (icon);
 		if (g_str_has_prefix (icon_name, "/")) {
 			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_INVALID,
@@ -744,6 +753,7 @@ as_app_validate_icons (AsApp *app, AsAppValidateHelper *helper)
 		}
 		break;
 	case AS_ICON_KIND_REMOTE:
+		icon_name = as_icon_get_url (icon);
 		if (!g_str_has_prefix (icon_name, "http://") &&
 		    !g_str_has_prefix (icon_name, "https://")) {
 			ai_app_validate_add (helper,
@@ -776,7 +786,11 @@ as_app_validate_screenshots (AsApp *app, AsAppValidateHelper *helper)
 		number_screenshots_min = 0;
 	}
 
-	/* metainfo and firmware do not require any screenshots */
+	/* firmware does not need screenshots */
+	if (as_app_get_kind (app) == AS_APP_KIND_FIRMWARE)
+		number_screenshots_min = 0;
+
+	/* metainfo and inf do not require any screenshots */
 	if (as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO ||
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_INF)
 		number_screenshots_min = 0;
@@ -820,7 +834,10 @@ as_app_validate_screenshots (AsApp *app, AsAppValidateHelper *helper)
  * as_app_validate_release:
  **/
 static gboolean
-as_app_validate_release (AsRelease *release, AsAppValidateHelper *helper, GError **error)
+as_app_validate_release (AsApp *app,
+			 AsRelease *release,
+			 AsAppValidateHelper *helper,
+			 GError **error)
 {
 	const gchar *tmp;
 	guint64 timestamp;
@@ -853,6 +870,14 @@ as_app_validate_release (AsRelease *release, AsAppValidateHelper *helper, GError
 				     "<release> timestamp should be a UNIX time");
 	}
 
+	/* for firmware, check urgency */
+	if (as_app_get_kind (app) == AS_APP_KIND_FIRMWARE &&
+	    as_release_get_urgency (release) == AS_URGENCY_KIND_UNKNOWN) {
+		ai_app_validate_add (helper,
+				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
+				     "<release> urgency is required for firmware");
+	}
+
 	/* check description */
 	tmp = as_release_get_description (release, "C");
 	if (tmp != NULL) {
@@ -885,7 +910,8 @@ as_app_validate_releases (AsApp *app, AsAppValidateHelper *helper, GError **erro
 	guint i;
 
 	/* only for AppData */
-	if (as_app_get_source_kind (app) != AS_APP_SOURCE_KIND_APPDATA)
+	if (as_app_get_source_kind (app) != AS_APP_SOURCE_KIND_APPDATA &&
+	    as_app_get_source_kind (app) != AS_APP_SOURCE_KIND_METAINFO)
 		return TRUE;
 
 	releases = as_app_get_releases (app);
@@ -896,7 +922,7 @@ as_app_validate_releases (AsApp *app, AsAppValidateHelper *helper, GError **erro
 	}
 	for (i = 0; i < releases->len; i++) {
 		release = g_ptr_array_index (releases, i);
-		if (!as_app_validate_release (release, helper, error))
+		if (!as_app_validate_release (app, release, helper, error))
 			return FALSE;
 	}
 	return TRUE;
@@ -908,11 +934,11 @@ as_app_validate_releases (AsApp *app, AsAppValidateHelper *helper, GError **erro
 static gboolean
 as_app_validate_setup_networking (AsAppValidateHelper *helper, GError **error)
 {
-	helper->session = soup_session_sync_new_with_options (SOUP_SESSION_USER_AGENT,
-							      "libappstream-glib",
-							      SOUP_SESSION_TIMEOUT,
-							      5000,
-							      NULL);
+	helper->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT,
+							 "libappstream-glib",
+							 SOUP_SESSION_TIMEOUT,
+							 5000,
+							 NULL);
 	if (helper->session == NULL) {
 		g_set_error_literal (error,
 				     AS_APP_ERROR,
@@ -932,9 +958,17 @@ static gboolean
 as_app_validate_license (const gchar *license_text, GError **error)
 {
 	guint i;
-	_cleanup_strv_free_ gchar **licenses = NULL;
+	g_auto(GStrv) licenses = NULL;
 
 	licenses = as_utils_spdx_license_tokenize (license_text);
+	if (licenses == NULL) {
+		g_set_error (error,
+			     AS_APP_ERROR,
+			     AS_APP_ERROR_FAILED,
+			     "SPDX license text '%s' could not be parsed",
+			     license_text);
+		return FALSE;
+	}
 	for (i = 0; licenses[i] != NULL; i++) {
 		if (g_strcmp0 (licenses[i], "&") == 0 ||
 		    g_strcmp0 (licenses[i], "|") == 0 ||
@@ -954,6 +988,45 @@ as_app_validate_license (const gchar *license_text, GError **error)
 	return TRUE;
 }
 
+/**
+ * as_app_validate_is_content_license_id:
+ **/
+static gboolean
+as_app_validate_is_content_license_id (const gchar *license_id)
+{
+	if (g_strcmp0 (license_id, "@CC0-1.0") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-3.0") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-3.0+") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-4.0") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-4.0+") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-SA-3.0") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-SA-3.0+") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-SA-4.0") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@CC-BY-SA-4.0+") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@GFDL-1.1") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@GFDL-1.2") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@GFDL-1.3") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@FSFAP") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "&") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "|") == 0)
+		return TRUE;
+	return FALSE;
+}
+
 /**
  * as_app_validate_is_content_license:
  **/
@@ -961,28 +1034,13 @@ static gboolean
 as_app_validate_is_content_license (const gchar *license)
 {
 	guint i;
-	_cleanup_strv_free_ gchar **tokens = NULL;
+	g_auto(GStrv) tokens = NULL;
 	tokens = as_utils_spdx_license_tokenize (license);
-	for (i = 0; tokens[i] != NULL; i++) {
-		if (g_strcmp0 (tokens[i], "@CC0-1.0") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "@CC-BY-3.0") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "@CC-BY-4.0") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "@CC-BY-SA-3.0") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "@CC-BY-SA-4.0") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "@GFDL-1.1") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "@GFDL-1.2") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "@GFDL-1.3") == 0)
-			continue;
-		if (g_strcmp0 (tokens[i], "&") == 0)
-			continue;
+	if (tokens == NULL)
 		return FALSE;
+	for (i = 0; tokens[i] != NULL; i++) {
+		if (!as_app_validate_is_content_license_id (tokens[i]))
+			return FALSE;
 	}
 	return TRUE;
 }
@@ -1017,6 +1075,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	const gchar *tmp;
 	const gchar *update_contact;
 	gboolean deprectated_failure = FALSE;
+	gboolean require_appstream_spec_only = FALSE;
 	gboolean require_contactdetails = TRUE;
 	gboolean require_copyright = FALSE;
 	gboolean require_project_license = FALSE;
@@ -1025,6 +1084,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	gboolean require_url = TRUE;
 	gboolean require_content_license = TRUE;
 	gboolean require_name = TRUE;
+	gboolean require_translation = TRUE;
 	gboolean validate_license = TRUE;
 	gboolean ret;
 	guint length_name_max = 30;
@@ -1034,7 +1094,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	guint number_para_max = 4;
 	guint number_para_min = 2;
 	guint str_len;
-	_cleanup_list_free_ GList *keys = NULL;
+	g_autoptr(GList) keys = NULL;
 
 	/* relax the requirements a bit */
 	if ((flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
@@ -1047,6 +1107,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		number_para_max = 10;
 		number_para_min = 1;
 		require_sentence_case = FALSE;
+		require_translation = FALSE;
 		switch (as_app_get_source_kind (app)) {
 		case AS_APP_SOURCE_KIND_METAINFO:
 		case AS_APP_SOURCE_KIND_APPDATA:
@@ -1064,6 +1125,17 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		require_translations = TRUE;
 		require_project_license = TRUE;
 		require_content_license = TRUE;
+		require_appstream_spec_only = TRUE;
+	}
+
+	/* addons don't need such a long description */
+	switch (as_app_get_source_kind (app)) {
+	case AS_APP_SOURCE_KIND_METAINFO:
+	case AS_APP_SOURCE_KIND_APPDATA:
+		number_para_min = 1;
+		break;
+	default:
+		break;
 	}
 
 	/* set up networking */
@@ -1084,35 +1156,35 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	/* id */
 	ret = FALSE;
 	id = as_app_get_id (app);
-	switch (as_app_get_id_kind (app)) {
-	case AS_ID_KIND_DESKTOP:
-	case AS_ID_KIND_WEB_APP:
+	switch (as_app_get_kind (app)) {
+	case AS_APP_KIND_DESKTOP:
+	case AS_APP_KIND_WEB_APP:
 		if (g_str_has_suffix (id, ".desktop"))
 			ret = TRUE;
 		break;
-	case AS_ID_KIND_INPUT_METHOD:
+	case AS_APP_KIND_INPUT_METHOD:
 		if (g_str_has_suffix (id, ".xml"))
 			ret = TRUE;
 		else if (g_str_has_suffix (id, ".db"))
 			ret = TRUE;
 		break;
-	case AS_ID_KIND_CODEC:
+	case AS_APP_KIND_CODEC:
 		if (g_str_has_prefix (id, "gstreamer"))
 			ret = TRUE;
 		break;
-	case AS_ID_KIND_UNKNOWN:
+	case AS_APP_KIND_UNKNOWN:
 		ai_app_validate_add (&helper,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
 				     "<id> has invalid type attribute");
 
 		break;
-	case AS_ID_KIND_FIRMWARE:
+	case AS_APP_KIND_FIRMWARE:
 		if (g_str_has_suffix (id, ".firmware"))
 			ret = TRUE;
 		break;
-	case AS_ID_KIND_FONT:
-	case AS_ID_KIND_ADDON:
-	case AS_ID_KIND_SOURCE:
+	case AS_APP_KIND_FONT:
+	case AS_APP_KIND_ADDON:
+	case AS_APP_KIND_SOURCE:
 		/* anything goes */
 		ret = TRUE;
 	default:
@@ -1186,6 +1258,15 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		}
 	}
 
+	/* translation */
+	if (require_translation &&
+	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_APPDATA &&
+	    as_app_get_translations (app)->len == 0) {
+		ai_app_validate_add (&helper,
+				     AS_PROBLEM_KIND_TAG_MISSING,
+				     "<translation> not specified");
+	}
+
 	/* pkgname */
 	if (as_app_get_pkgname_default (app) != NULL &&
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
@@ -1194,9 +1275,17 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 				     "<pkgname> not allowed in metainfo");
 	}
 
+	/* appdata */
+	if (as_app_get_icon_default (app) != NULL &&
+	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_APPDATA) {
+		ai_app_validate_add (&helper,
+				     AS_PROBLEM_KIND_TAG_INVALID,
+				     "<icon> not allowed in appdata");
+	}
+
 	/* extends */
 	if (as_app_get_extends(app)->len == 0 &&
-	    as_app_get_id_kind (app) == AS_ID_KIND_ADDON &&
+	    as_app_get_kind (app) == AS_APP_KIND_ADDON &&
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
 		ai_app_validate_add (&helper,
 				     AS_PROBLEM_KIND_TAG_MISSING,
@@ -1245,6 +1334,27 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 					     AS_PROBLEM_KIND_VALUE_MISSING,
 					     "<!-- Copyright [year] [name] --> is not present");
 		}
+		if (deprectated_failure &&
+		    (problems & AS_APP_PROBLEM_UPDATECONTACT_FALLBACK) > 0) {
+			ai_app_validate_add (&helper,
+					     AS_PROBLEM_KIND_TAG_INVALID,
+					     "<updatecontact> should be <update_contact>");
+		}
+	}
+
+	/* check invalid values */
+	if ((problems & AS_APP_PROBLEM_INVALID_PROJECT_GROUP) > 0) {
+		ai_app_validate_add (&helper,
+				     AS_PROBLEM_KIND_TAG_INVALID,
+				     "<project_group> is not valid");
+	}
+
+	/* only allow XML in the specification */
+	if (require_appstream_spec_only &&
+	    (problems & AS_APP_PROBLEM_INVALID_XML_TAG) > 0) {
+		ai_app_validate_add (&helper,
+				     AS_PROBLEM_KIND_TAG_INVALID,
+				     "XML data contains unknown tag");
 	}
 
 	/* check for things that have to exist */
