@@ -34,14 +34,12 @@
 
 #include "config.h"
 
-#include "as-cleanup.h"
 #include "as-icon-private.h"
 #include "as-node-private.h"
 #include "as-utils-private.h"
 #include "as-yaml.h"
 
-typedef struct _AsIconPrivate	AsIconPrivate;
-struct _AsIconPrivate
+typedef struct
 {
 	AsIconKind		 kind;
 	gchar			*name;
@@ -53,7 +51,7 @@ struct _AsIconPrivate
 	guint			 height;
 	GdkPixbuf		*pixbuf;
 	GBytes			*data;
-};
+} AsIconPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsIcon, as_icon, G_TYPE_OBJECT)
 
@@ -66,14 +64,7 @@ G_DEFINE_TYPE_WITH_PRIVATE (AsIcon, as_icon, G_TYPE_OBJECT)
  *
  * Since: 0.3.1
  **/
-GQuark
-as_icon_error_quark (void)
-{
-	static GQuark quark = 0;
-	if (!quark)
-		quark = g_quark_from_static_string ("AsIconError");
-	return quark;
-}
+G_DEFINE_QUARK (as-icon-error-quark, as_icon_error)
 
 /**
  * as_icon_finalize:
@@ -454,16 +445,11 @@ void
 as_icon_set_pixbuf (AsIcon *icon, GdkPixbuf *pixbuf)
 {
 	AsIconPrivate *priv = GET_PRIVATE (icon);
-
-	if (priv->pixbuf != NULL)
-		g_object_unref (priv->pixbuf);
-	if (pixbuf == NULL) {
-		priv->pixbuf = NULL;
-		return;
+	g_set_object (&priv->pixbuf, pixbuf);
+	if (pixbuf != NULL) {
+		priv->width = gdk_pixbuf_get_width (pixbuf);
+		priv->height = gdk_pixbuf_get_height (pixbuf);
 	}
-	priv->pixbuf = g_object_ref (pixbuf);
-	priv->width = gdk_pixbuf_get_width (pixbuf);
-	priv->height = gdk_pixbuf_get_height (pixbuf);
 }
 
 /**
@@ -497,7 +483,7 @@ as_icon_node_insert_embedded (AsIcon *icon, GNode *parent, AsNodeContext *ctx)
 {
 	AsIconPrivate *priv = GET_PRIVATE (icon);
 	GNode *n;
-	_cleanup_free_ gchar *data = NULL;
+	g_autofree gchar *data = NULL;
 
 	/* embedded icon */
 	n = as_node_insert (parent, "icon", NULL, 0,
@@ -574,9 +560,9 @@ as_icon_node_parse_embedded (AsIcon *icon, GNode *n, GError **error)
 	AsIconPrivate *priv = GET_PRIVATE (icon);
 	GNode *c;
 	gsize size;
-	_cleanup_free_ guchar *data = NULL;
-	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
-	_cleanup_object_unref_ GInputStream *stream = NULL;
+	g_autofree guchar *data = NULL;
+	g_autoptr(GdkPixbuf) pixbuf = NULL;
+	g_autoptr(GInputStream) stream = NULL;
 
 	/* get the icon name */
 	c = as_node_find (n, "name");
@@ -665,7 +651,7 @@ as_icon_node_parse (AsIcon *icon, GNode *node,
 		if (g_strstr_len (tmp, -1, "/") == NULL) {
 			as_icon_set_name (icon, tmp);
 		} else {
-			_cleanup_free_ gchar *basename = NULL;
+			g_autofree gchar *basename = NULL;
 			basename = g_path_get_basename (tmp);
 			as_icon_set_name (icon, basename);
 		}
@@ -717,10 +703,13 @@ gboolean
 as_icon_node_parse_dep11 (AsIcon *im, GNode *node,
 			  AsNodeContext *ctx, GError **error)
 {
-	if (g_strcmp0 (as_yaml_node_get_key (node), "cached") != 0)
-		return TRUE;
-	as_icon_set_name (im, as_yaml_node_get_value (node));
-	as_icon_set_kind (im, AS_ICON_KIND_CACHED);
+	if (g_strcmp0 (as_yaml_node_get_key (node), "cached") == 0) {
+		as_icon_set_name (im, as_yaml_node_get_value (node));
+		as_icon_set_kind (im, AS_ICON_KIND_CACHED);
+	} else if (g_strcmp0 (as_yaml_node_get_key (node), "stock") == 0) {
+		as_icon_set_name (im, as_yaml_node_get_value (node));
+		as_icon_set_kind (im, AS_ICON_KIND_STOCK);
+	}
 	return TRUE;
 }
 
@@ -740,8 +729,8 @@ gboolean
 as_icon_load (AsIcon *icon, AsIconLoadFlags flags, GError **error)
 {
 	AsIconPrivate *priv = GET_PRIVATE (icon);
-	_cleanup_free_ gchar *fn_fallback = NULL;
-	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
+	g_autofree gchar *fn_fallback = NULL;
+	g_autoptr(GdkPixbuf) pixbuf = NULL;
 
 	/* absolute filename */
 	if (priv->kind == AS_ICON_KIND_LOCAL) {
@@ -779,8 +768,8 @@ as_icon_load (AsIcon *icon, AsIconLoadFlags flags, GError **error)
 		guint height[] = { priv->height, 64, 128, 0 };
 		guint i;
 		for (i = 0; widths[i] != 0; i++) {
-			_cleanup_free_ gchar *fn_size = NULL;
-			_cleanup_free_ gchar *size_str = NULL;
+			g_autofree gchar *fn_size = NULL;
+			g_autofree gchar *size_str = NULL;
 			size_str = g_strdup_printf ("%ix%i", widths[i], height[i]);
 			fn_size = g_build_filename (priv->prefix, size_str, priv->name, NULL);
 			if (g_file_test (fn_size, G_FILE_TEST_EXISTS)) {
@@ -831,8 +820,8 @@ as_icon_convert_to_kind (AsIcon *icon, AsIconKind kind, GError **error)
 	/* cached -> embedded */
 	if (priv->kind == AS_ICON_KIND_CACHED && kind == AS_ICON_KIND_EMBEDDED) {
 		gsize data_size;
-		_cleanup_bytes_unref_ GBytes *tmp = NULL;
-		_cleanup_free_ gchar *data = NULL;
+		g_autoptr(GBytes) tmp = NULL;
+		g_autofree gchar *data = NULL;
 
 		/* load the pixbuf and save it to a PNG buffer */
 		if (priv->pixbuf == NULL) {
@@ -852,9 +841,9 @@ as_icon_convert_to_kind (AsIcon *icon, AsIconKind kind, GError **error)
 
 	/* cached -> embedded */
 	if (priv->kind == AS_ICON_KIND_EMBEDDED && kind == AS_ICON_KIND_CACHED) {
-		_cleanup_free_ gchar *size_str = NULL;
-		_cleanup_free_ gchar *path = NULL;
-		_cleanup_free_ gchar *fn = NULL;
+		g_autofree gchar *size_str = NULL;
+		g_autofree gchar *path = NULL;
+		g_autofree gchar *fn = NULL;
 
 		/* ensure the parent path exists */
 		size_str = g_strdup_printf ("%ix%i", priv->width, priv->height);
