@@ -29,22 +29,10 @@
 #include <glib-object.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#define AS_TYPE_IMAGE		(as_image_get_type())
-#define AS_IMAGE(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), AS_TYPE_IMAGE, AsImage))
-#define AS_IMAGE_CLASS(cls)	(G_TYPE_CHECK_CLASS_CAST((cls), AS_TYPE_IMAGE, AsImageClass))
-#define AS_IS_IMAGE(obj)	(G_TYPE_CHECK_INSTANCE_TYPE((obj), AS_TYPE_IMAGE))
-#define AS_IS_IMAGE_CLASS(cls)	(G_TYPE_CHECK_CLASS_TYPE((cls), AS_TYPE_IMAGE))
-#define AS_IMAGE_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), AS_TYPE_IMAGE, AsImageClass))
-
 G_BEGIN_DECLS
 
-typedef struct _AsImage		AsImage;
-typedef struct _AsImageClass	AsImageClass;
-
-struct _AsImage
-{
-	GObject			parent;
-};
+#define AS_TYPE_IMAGE (as_image_get_type ())
+G_DECLARE_DERIVABLE_TYPE (AsImage, as_image, AS, IMAGE, GObject)
 
 struct _AsImageClass
 {
@@ -94,6 +82,26 @@ typedef enum {
 	AS_IMAGE_SAVE_FLAG_LAST
 } AsImageSaveFlags;
 
+/**
+ * AsImageLoadFlags:
+ * @AS_IMAGE_LOAD_FLAG_NONE:		No special flags set
+ * @AS_IMAGE_LOAD_FLAG_SHARPEN:		Sharpen the resulting image
+ * @AS_IMAGE_LOAD_FLAG_SET_BASENAME:	Set the image basename
+ * @AS_IMAGE_LOAD_FLAG_SET_CHECKSUM:	Set the image checksum
+ * @AS_IMAGE_LOAD_FLAG_ONLY_SUPPORTED:	Only load supported formats like PNG and JPG
+ *
+ * The flags used for loading images.
+ **/
+typedef enum {
+	AS_IMAGE_LOAD_FLAG_NONE		= 0,	/* Since: 0.5.6 */
+	AS_IMAGE_LOAD_FLAG_SHARPEN	= 1,	/* Since: 0.5.6 */
+	AS_IMAGE_LOAD_FLAG_SET_BASENAME	= 2,	/* Since: 0.5.6 */
+	AS_IMAGE_LOAD_FLAG_SET_CHECKSUM	= 4,	/* Since: 0.5.6 */
+	AS_IMAGE_LOAD_FLAG_ONLY_SUPPORTED = 8,	/* Since: 0.5.6 */
+	/*< private >*/
+	AS_IMAGE_LOAD_FLAG_LAST
+} AsImageLoadFlags;
+
 /**
  * AsImageAlphaFlags:
  * @AS_IMAGE_ALPHA_FLAG_NONE:		No padding detected
@@ -124,7 +132,6 @@ typedef enum {
 #define AS_IMAGE_THUMBNAIL_HEIGHT	63	/* Since: 0.2.2 */
 #define AS_IMAGE_THUMBNAIL_WIDTH 	112	/* Since: 0.2.2 */
 
-GType		 as_image_get_type		(void);
 AsImage		*as_image_new			(void);
 
 /* helpers */
@@ -159,6 +166,12 @@ AsImageAlphaFlags as_image_get_alpha_flags	(AsImage	*image);
 gboolean	 as_image_load_filename		(AsImage	*image,
 						 const gchar	*filename,
 						 GError		**error);
+gboolean	 as_image_load_filename_full	(AsImage	*image,
+						 const gchar	*filename,
+						 guint		 dest_size,
+						 guint		 src_size_min,
+						 AsImageLoadFlags flags,
+						 GError		**error);
 GdkPixbuf	*as_image_save_pixbuf		(AsImage	*image,
 						 guint		 width,
 						 guint		 height,
@@ -169,6 +182,8 @@ gboolean	 as_image_save_filename		(AsImage	*image,
 						 guint		 height,
 						 AsImageSaveFlags flags,
 						 GError		**error);
+gboolean	 as_image_equal			(AsImage	*image1,
+						 AsImage	*image2);
 
 G_END_DECLS
 
