@@ -37,7 +37,7 @@
 #include <glib.h>
 #include <string.h>
 
-#include "as-cleanup.h"
+#include "as-markup.h"
 #include "as-node-private.h"
 #include "as-utils-private.h"
 
@@ -64,7 +64,7 @@ typedef struct {
  *
  * Since: 0.1.0
  **/
-GNode *
+AsNode *
 as_node_new (void)
 {
 	AsNodeData *data;
@@ -131,7 +131,7 @@ as_node_attr_lookup (AsNodeData *data, const gchar *key)
  * as_node_destroy_node_cb:
  **/
 static gboolean
-as_node_destroy_node_cb (GNode *node, gpointer user_data)
+as_node_destroy_node_cb (AsNode *node, gpointer user_data)
 {
 	AsNodeData *data = node->data;
 	if (data == NULL)
@@ -145,14 +145,14 @@ as_node_destroy_node_cb (GNode *node, gpointer user_data)
 
 /**
  * as_node_unref:
- * @node: a #GNode.
+ * @node: a #AsNode.
  *
  * Deallocates all notes in the tree.
  *
  * Since: 0.1.0
  **/
 void
-as_node_unref (GNode *node)
+as_node_unref (AsNode *node)
 {
 	g_node_traverse (node,
 			 G_PRE_ORDER,
@@ -170,33 +170,7 @@ as_node_unref (GNode *node)
  *
  * Since: 0.1.0
  **/
-GQuark
-as_node_error_quark (void)
-{
-	static GQuark quark = 0;
-	if (!quark)
-		quark = g_quark_from_static_string ("AsNodeError");
-	return quark;
-}
-
-/**
- * as_node_string_replace:
- **/
-static void
-as_node_string_replace (GString *string, const gchar *search, const gchar *replace)
-{
-	_cleanup_free_ gchar *tmp = NULL;
-	_cleanup_strv_free_ gchar **split = NULL;
-
-	/* quick search */
-	if (g_strstr_len (string->str, -1, search) == NULL)
-		return;
-
-	/* replace */
-	split = g_strsplit (string->str, search, -1);
-	tmp = g_strjoinv (replace, split);
-	g_string_assign (string, tmp);
-}
+G_DEFINE_QUARK (as-node-error-quark, as_node_error)
 
 /**
  * as_node_string_replace_inplace:
@@ -248,9 +222,9 @@ as_node_cdata_to_escaped (AsNodeData *data)
 		return;
 	str = g_string_new (data->cdata);
 	g_free (data->cdata);
-	as_node_string_replace (str, "&", "&amp;");
-	as_node_string_replace (str, "<", "&lt;");
-	as_node_string_replace (str, ">", "&gt;");
+	as_utils_string_replace (str, "&", "&amp;");
+	as_utils_string_replace (str, "<", "&lt;");
+	as_utils_string_replace (str, ">", "&gt;");
 	data->cdata = g_string_free (str, FALSE);
 	data->cdata_escaped = TRUE;
 }
@@ -320,11 +294,11 @@ as_node_data_set_name (AsNodeData *data, const gchar *name, AsNodeInsertFlags fl
  * as_node_sort_children:
  **/
 static void
-as_node_sort_children (GNode *first)
+as_node_sort_children (AsNode *first)
 {
 	AsNodeData *d1;
 	AsNodeData *d2;
-	GNode *child;
+	AsNode *child;
 	gpointer tmp;
 
 	d1 = (AsNodeData *) first->data;
@@ -351,11 +325,11 @@ as_node_sort_children (GNode *first)
 static void
 as_node_to_xml_string (GString *xml,
 		       guint depth_offset,
-		       const GNode *n,
+		       const AsNode *n,
 		       AsNodeToXmlFlags flags)
 {
 	AsNodeData *data = n->data;
-	GNode *c;
+	AsNode *c;
 	const gchar *tag_str;
 	const gchar *comment;
 	guint depth = g_node_depth ((GNode *) n);
@@ -365,7 +339,7 @@ as_node_to_xml_string (GString *xml,
 	comment = as_node_get_comment (n);
 	if (comment != NULL) {
 		guint i;
-		_cleanup_strv_free_ gchar **split = NULL;
+		g_auto(GStrv) split = NULL;
 
 		/* do not put additional spacing for the root node */
 		if (depth_offset < g_node_depth ((GNode *) n) &&
@@ -453,7 +427,7 @@ as_node_reflow_text (const gchar *text, gssize text_len)
 	GString *tmp;
 	guint i;
 	guint newline_count = 0;
-	_cleanup_strv_free_ gchar **split = NULL;
+	g_auto(GStrv) split = NULL;
 
 	/* split the text into lines */
 	tmp = g_string_sized_new (text_len + 1);
@@ -491,13 +465,13 @@ as_node_reflow_text (const gchar *text, gssize text_len)
 }
 
 typedef struct {
-	GNode			*current;
+	AsNode			*current;
 	AsNodeFromXmlFlags	 flags;
 } AsNodeToXmlHelper;
 
 /**
  * as_node_to_xml:
- * @node: a #GNode.
+ * @node: a #AsNode.
  * @flags: the AsNodeToXmlFlags, e.g. %AS_NODE_INSERT_FLAG_PRE_ESCAPED.
  *
  * Converts a node and it's children to XML.
@@ -507,10 +481,10 @@ typedef struct {
  * Since: 0.1.0
  **/
 GString *
-as_node_to_xml (const GNode *node, AsNodeToXmlFlags flags)
+as_node_to_xml (const AsNode *node, AsNodeToXmlFlags flags)
 {
 	GString *xml;
-	const GNode *l;
+	const AsNode *l;
 	guint depth_offset;
 
 	g_return_val_if_fail (node != NULL, NULL);
@@ -541,7 +515,7 @@ as_node_start_element_cb (GMarkupParseContext *context,
 {
 	AsNodeToXmlHelper *helper = (AsNodeToXmlHelper *) user_data;
 	AsNodeData *data;
-	GNode *current;
+	AsNode *current;
 	gchar *tmp;
 	guint i;
 
@@ -639,7 +613,7 @@ as_node_passthrough_cb (GMarkupParseContext *context,
 	const gchar *existing;
 	const gchar *tmp;
 	gchar *found;
-	_cleanup_free_ gchar *text = NULL;
+	g_autofree gchar *text = NULL;
 
 	/* only keep comments when told to */
 	if ((helper->flags & AS_NODE_FROM_XML_FLAG_KEEP_COMMENTS) == 0)
@@ -669,7 +643,7 @@ as_node_passthrough_cb (GMarkupParseContext *context,
 	if (existing == NULL) {
 		as_node_add_attribute (helper->current, "@comment-tmp", tmp);
 	} else {
-		_cleanup_free_ gchar *join = NULL;
+		g_autofree gchar *join = NULL;
 		join = g_strdup_printf ("%s<&>%s", existing, tmp);
 		as_node_add_attribute (helper->current, "@comment-tmp", join);
 	}
@@ -683,20 +657,20 @@ as_node_passthrough_cb (GMarkupParseContext *context,
  *
  * Parses XML data into a DOM tree.
  *
- * Returns: (transfer none): A populated #GNode tree
+ * Returns: (transfer none): A populated #AsNode tree
  *
  * Since: 0.1.0
  **/
-GNode *
+AsNode *
 as_node_from_xml (const gchar *data,
 		  AsNodeFromXmlFlags flags,
 		  GError **error)
 {
 	AsNodeToXmlHelper helper;
-	GNode *root = NULL;
+	AsNode *root = NULL;
 	gboolean ret;
-	_cleanup_error_free_ GError *error_local = NULL;
-	_cleanup_markup_parse_context_unref_ GMarkupParseContext *ctx = NULL;
+	g_autoptr(GError) error_local = NULL;
+	g_autoptr(GMarkupParseContext) ctx = NULL;
 	const GMarkupParser parser = {
 		as_node_start_element_cb,
 		as_node_end_element_cb,
@@ -704,7 +678,7 @@ as_node_from_xml (const gchar *data,
 		as_node_passthrough_cb,
 		NULL };
 
-	g_return_val_if_fail (data != NULL, FALSE);
+	g_return_val_if_fail (data != NULL, NULL);
 
 	root = as_node_new ();
 	helper.flags = flags;
@@ -740,7 +714,7 @@ as_node_from_xml (const gchar *data,
 
 /**
  * as_node_to_file: (skip)
- * @root: A populated #GNode tree
+ * @root: A populated #AsNode tree
  * @file: a #GFile
  * @flags: #AsNodeToXmlFlags, e.g. %AS_NODE_TO_XML_FLAG_NONE
  * @cancellable: A #GCancellable, or %NULL
@@ -753,13 +727,13 @@ as_node_from_xml (const gchar *data,
  * Since: 0.2.0
  **/
 gboolean
-as_node_to_file (const GNode *root,
+as_node_to_file (const AsNode *root,
 		 GFile *file,
 		 AsNodeToXmlFlags flags,
 		 GCancellable *cancellable,
 		 GError **error)
 {
-	_cleanup_string_free_ GString *xml = NULL;
+	g_autoptr(GString) xml = NULL;
 	xml = as_node_to_xml (root, flags);
 	return g_file_replace_contents (file,
 					xml->str,
@@ -781,11 +755,11 @@ as_node_to_file (const GNode *root,
  *
  * Parses an XML file into a DOM tree.
  *
- * Returns: (transfer none): A populated #GNode tree
+ * Returns: (transfer none): A populated #AsNode tree
  *
  * Since: 0.1.0
  **/
-GNode *
+AsNode *
 as_node_from_file (GFile *file,
 		   AsNodeFromXmlFlags flags,
 		   GCancellable *cancellable,
@@ -793,17 +767,17 @@ as_node_from_file (GFile *file,
 {
 	AsNodeToXmlHelper helper;
 	GError *error_local = NULL;
-	GNode *root = NULL;
+	AsNode *root = NULL;
 	const gchar *content_type = NULL;
 	gboolean ret = TRUE;
 	gsize chunk_size = 32 * 1024;
 	gssize len;
-	_cleanup_free_ gchar *data = NULL;
-	_cleanup_markup_parse_context_unref_ GMarkupParseContext *ctx = NULL;
-	_cleanup_object_unref_ GConverter *conv = NULL;
-	_cleanup_object_unref_ GFileInfo *info = NULL;
-	_cleanup_object_unref_ GInputStream *file_stream = NULL;
-	_cleanup_object_unref_ GInputStream *stream_data = NULL;
+	g_autofree gchar *data = NULL;
+	g_autoptr(GMarkupParseContext) ctx = NULL;
+	g_autoptr(GConverter) conv = NULL;
+	g_autoptr(GFileInfo) info = NULL;
+	g_autoptr(GInputStream) file_stream = NULL;
+	g_autoptr(GInputStream) stream_data = NULL;
 	const GMarkupParser parser = {
 		as_node_start_element_cb,
 		as_node_end_element_cb,
@@ -889,12 +863,12 @@ as_node_from_file (GFile *file,
 /**
  * as_node_get_child_node:
  **/
-static GNode *
-as_node_get_child_node (const GNode *root, const gchar *name,
+static AsNode *
+as_node_get_child_node (const AsNode *root, const gchar *name,
 			const gchar *attr_key, const gchar *attr_value)
 {
 	AsNodeData *data;
-	GNode *node;
+	AsNode *node;
 
 	/* invalid */
 	if (root == NULL)
@@ -922,7 +896,7 @@ as_node_get_child_node (const GNode *root, const gchar *name,
 
 /**
  * as_node_get_name:
- * @node: a #GNode
+ * @node: a #AsNode
  *
  * Gets the node name, e.g. "body"
  *
@@ -931,7 +905,7 @@ as_node_get_child_node (const GNode *root, const gchar *name,
  * Since: 0.1.0
  **/
 const gchar *
-as_node_get_name (const GNode *node)
+as_node_get_name (const AsNode *node)
 {
 	g_return_val_if_fail (node != NULL, NULL);
 	if (node->data == NULL)
@@ -941,7 +915,7 @@ as_node_get_name (const GNode *node)
 
 /**
  * as_node_set_name: (skip)
- * @node: a #GNode
+ * @node: a #AsNode
  * @name: the new name
  *
  * Sets the node name, e.g. "body"
@@ -949,7 +923,7 @@ as_node_get_name (const GNode *node)
  * Since: 0.1.4
  **/
 void
-as_node_set_name (GNode *node, const gchar *name)
+as_node_set_name (AsNode *node, const gchar *name)
 {
 	AsNodeData *data;
 
@@ -969,7 +943,7 @@ as_node_set_name (GNode *node, const gchar *name)
 
 /**
  * as_node_get_data:
- * @node: a #GNode
+ * @node: a #AsNode
  *
  * Gets the node data, e.g. "paragraph text"
  *
@@ -978,7 +952,7 @@ as_node_set_name (GNode *node, const gchar *name)
  * Since: 0.1.0
  **/
 const gchar *
-as_node_get_data (const GNode *node)
+as_node_get_data (const AsNode *node)
 {
 	AsNodeData *data;
 	g_return_val_if_fail (node != NULL, NULL);
@@ -993,7 +967,7 @@ as_node_get_data (const GNode *node)
 
 /**
  * as_node_get_comment:
- * @node: a #GNode
+ * @node: a #AsNode
  *
  * Gets the node data, e.g. "Copyright 2014 Richard Hughes"
  *
@@ -1002,14 +976,14 @@ as_node_get_data (const GNode *node)
  * Since: 0.1.6
  **/
 const gchar *
-as_node_get_comment (const GNode *node)
+as_node_get_comment (const AsNode *node)
 {
 	return as_node_get_attribute (node, "@comment");
 }
 
 /**
  * as_node_get_tag:
- * @node: a #GNode
+ * @node: a #AsNode
  *
  * Gets the node tag enum.
  *
@@ -1018,7 +992,7 @@ as_node_get_comment (const GNode *node)
  * Since: 0.1.2
  **/
 AsTag
-as_node_get_tag (const GNode *node)
+as_node_get_tag (const AsNode *node)
 {
 	AsNodeData *data;
 	const gchar *tmp;
@@ -1040,7 +1014,7 @@ as_node_get_tag (const GNode *node)
 
 /**
  * as_node_set_data: (skip)
- * @node: a #GNode
+ * @node: a #AsNode
  * @cdata: new data
  * @insert_flags: any %AsNodeInsertFlags.
  *
@@ -1049,7 +1023,7 @@ as_node_get_tag (const GNode *node)
  * Since: 0.1.1
  **/
 void
-as_node_set_data (GNode *node,
+as_node_set_data (AsNode *node,
 		  const gchar *cdata,
 		  AsNodeInsertFlags insert_flags)
 {
@@ -1068,7 +1042,7 @@ as_node_set_data (GNode *node,
 
 /**
  * as_node_set_comment: (skip)
- * @node: a #GNode
+ * @node: a #AsNode
  * @comment: new comment
  *
  * Sets new comment for the node.
@@ -1076,14 +1050,14 @@ as_node_set_data (GNode *node,
  * Since: 0.1.6
  **/
 void
-as_node_set_comment (GNode *node, const gchar *comment)
+as_node_set_comment (AsNode *node, const gchar *comment)
 {
-	return as_node_add_attribute (node, "@comment", comment);
+	as_node_add_attribute (node, "@comment", comment);
 }
 
 /**
  * as_node_take_data:
- * @node: a #GNode
+ * @node: a #AsNode
  *
  * Gets the node data, e.g. "paragraph text"
  *
@@ -1092,7 +1066,7 @@ as_node_set_comment (GNode *node, const gchar *comment)
  * Since: 0.1.0
  **/
 gchar *
-as_node_take_data (const GNode *node)
+as_node_take_data (const AsNode *node)
 {
 	AsNodeData *data;
 	gchar *tmp;
@@ -1110,7 +1084,7 @@ as_node_take_data (const GNode *node)
 
 /**
  * as_node_get_attribute_as_int:
- * @node: a #GNode
+ * @node: a #AsNode
  * @key: the attribute key
  *
  * Gets a node attribute, e.g. 34
@@ -1120,7 +1094,7 @@ as_node_take_data (const GNode *node)
  * Since: 0.1.0
  **/
 gint
-as_node_get_attribute_as_int (const GNode *node, const gchar *key)
+as_node_get_attribute_as_int (const AsNode *node, const gchar *key)
 {
 	const gchar *tmp;
 	gchar *endptr = NULL;
@@ -1139,7 +1113,7 @@ as_node_get_attribute_as_int (const GNode *node, const gchar *key)
 
 /**
  * as_node_get_attribute:
- * @node: a #GNode
+ * @node: a #AsNode
  * @key: the attribute key
  *
  * Gets a node attribute, e.g. "false"
@@ -1149,7 +1123,7 @@ as_node_get_attribute_as_int (const GNode *node, const gchar *key)
  * Since: 0.1.0
  **/
 const gchar *
-as_node_get_attribute (const GNode *node, const gchar *key)
+as_node_get_attribute (const AsNode *node, const gchar *key)
 {
 	AsNodeData *data;
 
@@ -1164,7 +1138,7 @@ as_node_get_attribute (const GNode *node, const gchar *key)
 
 /**
  * as_node_take_attribute: (skip)
- * @node: a #GNode
+ * @node: a #AsNode
  * @key: the attribute key
  *
  * Gets a node attribute, e.g. "false"
@@ -1174,7 +1148,7 @@ as_node_get_attribute (const GNode *node, const gchar *key)
  * Since: 0.1.2
  **/
 gchar *
-as_node_take_attribute (const GNode *node, const gchar *key)
+as_node_take_attribute (const AsNode *node, const gchar *key)
 {
 	AsNodeAttr *attr;
 	AsNodeData *data;
@@ -1196,7 +1170,7 @@ as_node_take_attribute (const GNode *node, const gchar *key)
 
 /**
  * as_node_remove_attribute: (skip)
- * @node: a #GNode
+ * @node: a #AsNode
  * @key: the attribute key
  *
  * Removes a node attribute, e.g. "type"
@@ -1204,7 +1178,7 @@ as_node_take_attribute (const GNode *node, const gchar *key)
  * Since: 0.2.0
  **/
 void
-as_node_remove_attribute (GNode *node, const gchar *key)
+as_node_remove_attribute (AsNode *node, const gchar *key)
 {
 	AsNodeAttr *attr;
 	AsNodeData *data;
@@ -1224,7 +1198,7 @@ as_node_remove_attribute (GNode *node, const gchar *key)
 
 /**
  * as_node_add_attribute: (skip)
- * @node: a #GNode
+ * @node: a #AsNode
  * @key: the attribute key
  * @value: new data
  *
@@ -1233,7 +1207,7 @@ as_node_remove_attribute (GNode *node, const gchar *key)
  * Since: 0.1.1
  **/
 void
-as_node_add_attribute (GNode *node,
+as_node_add_attribute (AsNode *node,
 		       const gchar *key,
 		       const gchar *value)
 {
@@ -1252,7 +1226,7 @@ as_node_add_attribute (GNode *node,
 
 /**
  * as_node_add_attribute_as_int: (skip)
- * @node: a #GNode
+ * @node: a #AsNode
  * @key: the attribute key
  * @value: new data
  *
@@ -1261,9 +1235,9 @@ as_node_add_attribute (GNode *node,
  * Since: 0.3.1
  **/
 void
-as_node_add_attribute_as_int (GNode *node, const gchar *key, gint value)
+as_node_add_attribute_as_int (AsNode *node, const gchar *key, gint value)
 {
-	_cleanup_free_ gchar *tmp = g_strdup_printf ("%i", value);
+	g_autofree gchar *tmp = g_strdup_printf ("%i", value);
 	as_node_add_attribute (node, key, tmp);
 }
 
@@ -1274,16 +1248,16 @@ as_node_add_attribute_as_int (GNode *node, const gchar *key, gint value)
  *
  * Gets a node from the DOM tree.
  *
- * Return value: A #GNode, or %NULL if not found
+ * Return value: A #AsNode, or %NULL if not found
  *
  * Since: 0.1.0
  **/
-GNode *
-as_node_find (GNode *root, const gchar *path)
+AsNode *
+as_node_find (AsNode *root, const gchar *path)
 {
-	GNode *node = root;
+	AsNode *node = root;
 	guint i;
-	_cleanup_strv_free_ gchar **split = NULL;
+	g_auto(GStrv) split = NULL;
 
 	g_return_val_if_fail (path != NULL, NULL);
 
@@ -1305,17 +1279,17 @@ as_node_find (GNode *root, const gchar *path)
  *
  * Gets a node from the DOM tree with a specified attribute.
  *
- * Return value: A #GNode, or %NULL if not found
+ * Return value: A #AsNode, or %NULL if not found
  *
  * Since: 0.1.0
  **/
-GNode *
-as_node_find_with_attribute (GNode *root, const gchar *path,
+AsNode *
+as_node_find_with_attribute (AsNode *root, const gchar *path,
 			     const gchar *attr_key, const gchar *attr_value)
 {
-	GNode *node = root;
+	AsNode *node = root;
 	guint i;
-	_cleanup_strv_free_ gchar **split = NULL;
+	g_auto(GStrv) split = NULL;
 
 	g_return_val_if_fail (path != NULL, NULL);
 
@@ -1361,7 +1335,7 @@ as_node_insert_line_breaks (const gchar *text, guint break_len)
 
 /**
  * as_node_insert: (skip)
- * @parent: a parent #GNode.
+ * @parent: a parent #AsNode.
  * @name: the tag name, e.g. "id".
  * @cdata: the tag data, or %NULL, e.g. "org.gnome.Software.desktop".
  * @insert_flags: any %AsNodeInsertFlags.
@@ -1369,12 +1343,12 @@ as_node_insert_line_breaks (const gchar *text, guint break_len)
  *
  * Inserts a node into the DOM.
  *
- * Returns: (transfer none): A populated #GNode
+ * Returns: (transfer none): A populated #AsNode
  *
  * Since: 0.1.0
  **/
-GNode *
-as_node_insert (GNode *parent,
+AsNode *
+as_node_insert (AsNode *parent,
 		const gchar *name,
 		const gchar *cdata,
 		AsNodeInsertFlags insert_flags,
@@ -1425,7 +1399,7 @@ as_node_list_sort_cb (gconstpointer a, gconstpointer b)
 
 /**
  * as_node_insert_localized:
- * @parent: a parent #GNode.
+ * @parent: a parent #AsNode.
  * @name: the tag name, e.g. "id".
  * @localized: the hash table of data, with the locale as the key.
  * @insert_flags: any %AsNodeInsertFlags.
@@ -1435,7 +1409,7 @@ as_node_list_sort_cb (gconstpointer a, gconstpointer b)
  * Since: 0.1.0
  **/
 void
-as_node_insert_localized (GNode *parent,
+as_node_insert_localized (AsNode *parent,
 			  const gchar *name,
 			  GHashTable *localized,
 			  AsNodeInsertFlags insert_flags)
@@ -1445,7 +1419,7 @@ as_node_insert_localized (GNode *parent,
 	const gchar *key;
 	const gchar *value;
 	const gchar *value_c;
-	_cleanup_list_free_ GList *list = NULL;
+	g_autoptr(GList) list = NULL;
 
 	g_return_if_fail (name != NULL);
 
@@ -1471,6 +1445,8 @@ as_node_insert_localized (GNode *parent,
 		key = l->data;
 		if (g_strcmp0 (key, "C") == 0)
 			continue;
+		if (g_strcmp0 (key, "x-test") == 0)
+			continue;
 		value = g_hash_table_lookup (localized, key);
 		if ((insert_flags & AS_NODE_INSERT_FLAG_DEDUPE_LANG) > 0 &&
 		    g_strcmp0 (value_c, value) == 0)
@@ -1491,7 +1467,7 @@ as_node_insert_localized (GNode *parent,
 
 /**
  * as_node_insert_hash:
- * @parent: a parent #GNode.
+ * @parent: a parent #AsNode.
  * @name: the tag name, e.g. "id".
  * @attr_key: the key to use as the attribute in the XML, e.g. "key".
  * @hash: the hash table with the key as the key to use in the XML.
@@ -1502,7 +1478,7 @@ as_node_insert_localized (GNode *parent,
  * Since: 0.1.0
  **/
 void
-as_node_insert_hash (GNode *parent,
+as_node_insert_hash (AsNode *parent,
 		     const gchar *name,
 		     const gchar *attr_key,
 		     GHashTable *hash,
@@ -1540,7 +1516,7 @@ as_node_insert_hash (GNode *parent,
 
 /**
  * as_node_get_localized:
- * @node: a #GNode
+ * @node: a #AsNode
  * @key: the key to use, e.g. "copyright"
  *
  * Extracts localized values from the DOM tree
@@ -1550,14 +1526,14 @@ as_node_insert_hash (GNode *parent,
  * Since: 0.1.0
  **/
 GHashTable *
-as_node_get_localized (const GNode *node, const gchar *key)
+as_node_get_localized (const AsNode *node, const gchar *key)
 {
 	AsNodeData *data;
 	const gchar *xml_lang;
 	const gchar *data_unlocalized;
 	const gchar *data_localized;
 	GHashTable *hash = NULL;
-	GNode *tmp;
+	AsNode *tmp;
 
 	/* does it exist? */
 	tmp = as_node_get_child_node (node, key, NULL, NULL);
@@ -1576,6 +1552,8 @@ as_node_get_localized (const GNode *node, const gchar *key)
 		if (g_strcmp0 (as_tag_data_get_name (data), key) != 0)
 			continue;
 		xml_lang = as_node_attr_lookup (data, "xml:lang");
+		if (g_strcmp0 (xml_lang, "x-test") == 0)
+			continue;
 
 		/* avoid storing identical strings */
 		data_localized = data->cdata;
@@ -1590,7 +1568,7 @@ as_node_get_localized (const GNode *node, const gchar *key)
 
 /**
  * as_node_get_localized_best:
- * @node: a #GNode.
+ * @node: a #AsNode.
  * @key: the tag name.
  *
  * Gets the 'best' locale version of a specific data value.
@@ -1600,9 +1578,9 @@ as_node_get_localized (const GNode *node, const gchar *key)
  * Since: 0.1.0
  **/
 const gchar *
-as_node_get_localized_best (const GNode *node, const gchar *key)
+as_node_get_localized_best (const AsNode *node, const gchar *key)
 {
-	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
+	g_autoptr(GHashTable) hash = NULL;
 	hash = as_node_get_localized (node, key);
 	if (hash == NULL)
 		return NULL;
@@ -1629,7 +1607,7 @@ as_node_denorm_add_to_langs (GHashTable *hash,
 	GList *l;
 	GString *str;
 	const gchar *xml_lang;
-	_cleanup_list_free_ GList *keys = NULL;
+	g_autoptr(GList) keys = NULL;
 
 	keys = g_hash_table_get_keys (hash);
 	for (l = keys; l != NULL; l = l->next) {
@@ -1682,12 +1660,12 @@ as_node_denorm_get_str_for_lang (GHashTable *hash,
  * ]|
  **/
 static gboolean
-as_node_get_localized_unwrap_type_li (const GNode *node,
+as_node_get_localized_unwrap_type_li (const AsNode *node,
 				      GHashTable *hash,
 				      GError **error)
 {
-	GNode *tmp;
-	GNode *tmp_c;
+	AsNode *tmp;
+	AsNode *tmp_c;
 	AsNodeData *data;
 	AsNodeData *data_c;
 	GString *str;
@@ -1770,12 +1748,12 @@ as_node_get_localized_unwrap_type_li (const GNode *node,
  * ]|
  **/
 static gboolean
-as_node_get_localized_unwrap_type_ul (const GNode *node,
+as_node_get_localized_unwrap_type_ul (const AsNode *node,
 				      GHashTable *hash,
 				      GError **error)
 {
-	GNode *tmp;
-	GNode *tmp_c;
+	AsNode *tmp;
+	AsNode *tmp_c;
 	AsNodeData *data;
 	AsNodeData *data_c;
 	GString *str;
@@ -1827,9 +1805,34 @@ as_node_get_localized_unwrap_type_ul (const GNode *node,
 	return TRUE;
 }
 
+/**
+ * as_node_fix_locale: (skip)
+ *
+ * Fixes and filters incorrect locale strings.
+ *
+ * Returns: a newly allocated string
+ *
+ * Since: 0.5.2
+ **/
+gchar *
+as_node_fix_locale (const gchar *locale)
+{
+	gchar *tmp;
+
+	if (locale == NULL)
+		return g_strdup ("C");
+	if (g_strcmp0 (locale, "xx") == 0)
+		return NULL;
+	if (g_strcmp0 (locale, "x-test") == 0)
+		return NULL;
+	tmp = g_strdup (locale);
+	g_strdelimit (tmp, "-", '_');
+	return tmp;
+}
+
 /**
  * as_node_get_localized_unwrap:
- * @node: a #GNode.
+ * @node: a #AsNode.
  * @error: A #GError or %NULL.
  *
  * Denormalize AppData data like this:
@@ -1857,17 +1860,17 @@ as_node_get_localized_unwrap_type_ul (const GNode *node,
  * Since: 0.1.0
  **/
 GHashTable *
-as_node_get_localized_unwrap (const GNode *node, GError **error)
+as_node_get_localized_unwrap (const AsNode *node, GError **error)
 {
 	AsNodeData *data;
 	GHashTable *results;
 	GList *l;
-	GNode *tmp;
+	AsNode *tmp;
 	GString *str;
 	const gchar *xml_lang;
 	gboolean is_li_translated = TRUE;
-	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
-	_cleanup_list_free_ GList *keys = NULL;
+	g_autoptr(GHashTable) hash = NULL;
+	g_autoptr(GList) keys = NULL;
 
 	g_return_val_if_fail (node != NULL, NULL);
 
@@ -1908,10 +1911,14 @@ as_node_get_localized_unwrap (const GNode *node, GError **error)
 	results = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	keys = g_hash_table_get_keys (hash);
 	for (l = keys; l != NULL; l = l->next) {
+		gchar *locale_fixed;
 		xml_lang = l->data;
+		locale_fixed = as_node_fix_locale (xml_lang);
+		if (locale_fixed == NULL)
+			continue;
 		str = g_hash_table_lookup (hash, xml_lang);
 		g_hash_table_insert (results,
-				     g_strdup (xml_lang),
+				     locale_fixed,
 				     g_strdup (str->str));
 	}
 	return results;
@@ -1922,6 +1929,8 @@ struct _AsNodeContext {
 	AsAppSourceKind	 source_kind;
 	AsAppSourceKind	 output;
 	gdouble		 version;
+	gboolean	 output_trusted;
+	gchar		*media_base_url;
 };
 
 /**
@@ -2006,6 +2015,37 @@ as_node_context_set_source_kind (AsNodeContext *ctx, AsAppSourceKind source_kind
 	ctx->source_kind = source_kind;
 }
 
+/**
+ * as_node_context_get_output_trusted: (skip)
+ * @ctx: a #AsNodeContext.
+ *
+ * Gets if the AppStream output is trusted to handle private data.
+ *
+ * Returns: output_trusted number
+ *
+ * Since: 0.5.5
+ **/
+gboolean
+as_node_context_get_output_trusted (AsNodeContext *ctx)
+{
+	return ctx->output_trusted;
+}
+
+/**
+ * as_node_context_set_output_trusted: (skip)
+ * @ctx: a #AsNodeContext.
+ * @output_trusted: boolean.
+ *
+ * Sets the AppStream output is trusted to handle private data.
+ *
+ * Since: 0.5.5
+ **/
+void
+as_node_context_set_output_trusted (AsNodeContext *ctx, gboolean output_trusted)
+{
+	ctx->output_trusted = output_trusted;
+}
+
 /**
  * as_node_context_get_output: (skip)
  * @ctx: a #AsNodeContext.
@@ -2036,3 +2076,33 @@ as_node_context_set_output (AsNodeContext *ctx, AsAppSourceKind output)
 {
 	ctx->output = output;
 }
+
+/**
+ * as_node_context_get_media_base_url: (skip)
+ * @ctx: a #AsNodeContext.
+ *
+ * Gets the base URL for media used when inserting nodes.
+ *
+ * Since: 0.5.11
+ **/
+const gchar *
+as_node_context_get_media_base_url (AsNodeContext *ctx)
+{
+	return ctx->media_base_url;
+}
+
+/**
+ * as_node_context_set_media_base_url: (skip)
+ * @ctx: a #AsNodeContext.
+ * @url: a URL
+ *
+ * Sets the base URL for media used when inserting nodes.
+ *
+ * Since: 0.5.11
+ **/
+void
+as_node_context_set_media_base_url (AsNodeContext *ctx, const gchar *url)
+{
+	g_free (ctx->media_base_url);
+	ctx->media_base_url = g_strdup (url);
+}
