@@ -33,7 +33,6 @@
 
 #include "config.h"
 
-#include "as-cleanup.h"
 #include "as-image-private.h"
 #include "as-node-private.h"
 #include "as-screenshot-private.h"
@@ -41,14 +40,13 @@
 #include "as-utils-private.h"
 #include "as-yaml.h"
 
-typedef struct _AsScreenshotPrivate	AsScreenshotPrivate;
-struct _AsScreenshotPrivate
+typedef struct
 {
 	AsScreenshotKind	 kind;
 	GHashTable		*captions;
 	GPtrArray		*images;
 	gint			 priority;
-};
+} AsScreenshotPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsScreenshot, as_screenshot, G_TYPE_OBJECT)
 
@@ -343,7 +341,7 @@ as_screenshot_set_caption (AsScreenshot *screenshot,
  *
  * Inserts the screenshot into the DOM tree.
  *
- * Returns: (transfer none): A populated #GNode
+ * Returns: (transfer none): A populated #GNode, or %NULL
  *
  * Since: 0.1.1
  **/
@@ -357,6 +355,10 @@ as_screenshot_node_insert (AsScreenshot *screenshot,
 	GNode *n;
 	guint i;
 
+	/* nothing to add */
+	if (priv->images->len == 0)
+		return NULL;
+
 	n = as_node_insert (parent, "screenshot", NULL,
 			    AS_NODE_INSERT_FLAG_NONE,
 			    NULL);
@@ -402,7 +404,7 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node,
 	const gchar *tmp;
 	guint size;
 	gint priority;
-	_cleanup_hashtable_unref_ GHashTable *captions = NULL;
+	g_autoptr(GHashTable) captions = NULL;
 
 	tmp = as_node_get_attribute (node, "type");
 	if (tmp != NULL) {
@@ -416,7 +418,7 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node,
 	/* add captions */
 	captions = as_node_get_localized (node, "caption");
 	if (captions != NULL) {
-		_cleanup_list_free_ GList *keys = NULL;
+		g_autoptr(GList) keys = NULL;
 		keys = g_hash_table_get_keys (captions);
 		for (l = keys; l != NULL; l = l->next) {
 			tmp = l->data;
@@ -444,7 +446,7 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node,
 
 	/* add images */
 	for (c = node->children; c != NULL; c = c->next) {
-		_cleanup_object_unref_ AsImage *image = NULL;
+		g_autoptr(AsImage) image = NULL;
 		if (as_node_get_tag (c) != AS_TAG_IMAGE)
 			continue;
 		image = as_image_new ();
@@ -486,7 +488,7 @@ as_screenshot_node_parse_dep11 (AsScreenshot *ss, GNode *node,
 			continue;
 		}
 		if (g_strcmp0 (tmp, "source-image") == 0) {
-			_cleanup_object_unref_ AsImage *im = as_image_new ();
+			g_autoptr(AsImage) im = as_image_new ();
 			as_image_set_kind (im, AS_IMAGE_KIND_SOURCE);
 			if (!as_image_node_parse_dep11 (im, n, ctx, error))
 				return FALSE;
@@ -495,7 +497,7 @@ as_screenshot_node_parse_dep11 (AsScreenshot *ss, GNode *node,
 		}
 		if (g_strcmp0 (tmp, "thumbnails") == 0) {
 			for (c = n->children; c != NULL; c = c->next) {
-				_cleanup_object_unref_ AsImage *im = as_image_new ();
+				g_autoptr(AsImage) im = as_image_new ();
 				as_image_set_kind (im, AS_IMAGE_KIND_THUMBNAIL);
 				if (!as_image_node_parse_dep11 (im, c, ctx, error))
 					return FALSE;
@@ -507,6 +509,50 @@ as_screenshot_node_parse_dep11 (AsScreenshot *ss, GNode *node,
 	return TRUE;
 }
 
+/**
+ * as_screenshot_equal:
+ * @screenshot1: a #AsScreenshot instance.
+ * @screenshot2: a #AsScreenshot instance.
+ *
+ * Checks if two screenshots are the same.
+ *
+ * Returns: %TRUE for success
+ *
+ * Since: 0.5.7
+ **/
+gboolean
+as_screenshot_equal (AsScreenshot *screenshot1, AsScreenshot *screenshot2)
+{
+	AsScreenshotPrivate *priv1 = GET_PRIVATE (screenshot1);
+	AsScreenshotPrivate *priv2 = GET_PRIVATE (screenshot2);
+	AsImage *im1;
+	AsImage *im2;
+
+	/* trivial */
+	if (screenshot1 == screenshot2)
+		return TRUE;
+
+	/* check for equality */
+	if (priv1->priority != priv2->priority)
+		return FALSE;
+	if (priv1->images->len != priv2->images->len)
+		return FALSE;
+	if (g_strcmp0 (as_screenshot_get_caption (screenshot1, NULL),
+		       as_screenshot_get_caption (screenshot2, NULL)) != 0)
+		return FALSE;
+
+	/* check source images */
+	im1 = as_screenshot_get_source (screenshot1);
+	im2 = as_screenshot_get_source (screenshot2);
+	if (im1 != NULL && im2 != NULL) {
+		if (!as_image_equal (im1, im2))
+			return FALSE;
+	}
+
+	/* success */
+	return TRUE;
+}
+
 /**
  * as_screenshot_new:
  *
