@@ -36,10 +36,11 @@
 #include <string.h>
 #include <archive_entry.h>
 #include <archive.h>
+#include <libsoup/soup.h>
 #include <stdlib.h>
+#include <uuid.h>
 
 #include "as-app.h"
-#include "as-cleanup.h"
 #include "as-enums.h"
 #include "as-node.h"
 #include "as-resources.h"
@@ -54,218 +55,7 @@
  *
  * Since: 0.3.7
  **/
-GQuark
-as_utils_error_quark (void)
-{
-	static GQuark quark = 0;
-	if (!quark)
-		quark = g_quark_from_static_string ("AsUtilsError");
-	return quark;
-}
-
-/**
- * as_markup_strsplit_words:
- * @text: the text to split.
- * @line_len: the maximum length of the output line
- *
- * Splits up a long line into an array of smaller strings, each being no longer
- * than @line_len. Words are not split.
- *
- * Returns: (transfer full): lines, or %NULL in event of an error
- *
- * Since: 0.3.5
- **/
-gchar **
-as_markup_strsplit_words (const gchar *text, guint line_len)
-{
-	GPtrArray *lines;
-	guint i;
-	_cleanup_string_free_ GString *curline = NULL;
-	_cleanup_strv_free_ gchar **tokens = NULL;
-
-	/* sanity check */
-	if (text == NULL || text[0] == '\0')
-		return NULL;
-	if (line_len == 0)
-		return NULL;
-
-	lines = g_ptr_array_new ();
-	curline = g_string_new ("");
-
-	/* tokenize the string */
-	tokens = g_strsplit (text, " ", -1);
-	for (i = 0; tokens[i] != NULL; i++) {
-
-		/* current line plus new token is okay */
-		if (curline->len + strlen (tokens[i]) < line_len) {
-			g_string_append_printf (curline, "%s ", tokens[i]);
-			continue;
-		}
-
-		/* too long, so remove space, add newline and dump */
-		if (curline->len > 0)
-			g_string_truncate (curline, curline->len - 1);
-		g_string_append (curline, "\n");
-		g_ptr_array_add (lines, g_strdup (curline->str));
-		g_string_truncate (curline, 0);
-		g_string_append_printf (curline, "%s ", tokens[i]);
-
-	}
-
-	/* any incomplete line? */
-	if (curline->len > 0) {
-		g_string_truncate (curline, curline->len - 1);
-		g_string_append (curline, "\n");
-		g_ptr_array_add (lines, g_strdup (curline->str));
-	}
-
-	g_ptr_array_add (lines, NULL);
-	return (gchar **) g_ptr_array_free (lines, FALSE);
-}
-
-/**
- * as_markup_render_para:
- **/
-static void
-as_markup_render_para (GString *str, AsMarkupConvertFormat format, const gchar *data)
-{
-	guint i;
-	_cleanup_strv_free_ gchar **spl = NULL;
-
-	if (str->len > 0)
-		g_string_append (str, "\n");
-	switch (format) {
-	case AS_MARKUP_CONVERT_FORMAT_SIMPLE:
-		g_string_append_printf (str, "%s\n", data);
-		break;
-	case AS_MARKUP_CONVERT_FORMAT_MARKDOWN:
-		/* break to 80 chars */
-		spl = as_markup_strsplit_words (data, 80);
-		for (i = 0; spl[i] != NULL; i++)
-			g_string_append (str, spl[i]);
-		break;
-	default:
-		break;
-	}
-}
-
-/**
- * as_markup_render_li:
- **/
-static void
-as_markup_render_li (GString *str, AsMarkupConvertFormat format, const gchar *data)
-{
-	guint i;
-	_cleanup_strv_free_ gchar **spl = NULL;
-
-	switch (format) {
-	case AS_MARKUP_CONVERT_FORMAT_SIMPLE:
-		g_string_append_printf (str, " • %s\n", data);
-		break;
-	case AS_MARKUP_CONVERT_FORMAT_MARKDOWN:
-		/* break to 80 chars, leaving room for the dot/indent */
-		spl = as_markup_strsplit_words (data, 80 - 3);
-		g_string_append_printf (str, " * %s", spl[0]);
-		for (i = 1; spl[i] != NULL; i++)
-			g_string_append_printf (str, "   %s", spl[i]);
-		break;
-	default:
-		break;
-	}
-}
-
-/**
- * as_markup_convert:
- * @markup: the text to copy.
- * @format: the #AsMarkupConvertFormat, e.g. %AS_MARKUP_CONVERT_FORMAT_MARKDOWN
- * @error: A #GError or %NULL
- *
- * Converts an XML description into a printable form.
- *
- * Returns: (transfer full): a newly allocated %NULL terminated string
- *
- * Since: 0.3.5
- **/
-gchar *
-as_markup_convert (const gchar *markup,
-		   AsMarkupConvertFormat format, GError **error)
-{
-	GNode *tmp;
-	GNode *tmp_c;
-	const gchar *tag;
-	const gchar *tag_c;
-	_cleanup_node_unref_ GNode *root = NULL;
-	_cleanup_string_free_ GString *str = NULL;
-
-	/* is this actually markup */
-	if (g_strstr_len (markup, -1, "<") == NULL)
-		return g_strdup (markup);
-
-	/* load */
-	root = as_node_from_xml (markup, AS_NODE_FROM_XML_FLAG_NONE, error);
-	if (root == NULL)
-		return NULL;
-
-	/* format */
-	str = g_string_new ("");
-	for (tmp = root->children; tmp != NULL; tmp = tmp->next) {
-
-		tag = as_node_get_name (tmp);
-		if (g_strcmp0 (tag, "p") == 0) {
-			as_markup_render_para (str, format, as_node_get_data (tmp));
-
-		/* loop on the children */
-		} else if (g_strcmp0 (tag, "ul") == 0 ||
-			   g_strcmp0 (tag, "ol") == 0) {
-			for (tmp_c = tmp->children; tmp_c != NULL; tmp_c = tmp_c->next) {
-				tag_c = as_node_get_name (tmp_c);
-				if (g_strcmp0 (tag_c, "li") == 0) {
-					as_markup_render_li (str, format,
-							     as_node_get_data (tmp_c));
-				} else {
-					/* only <li> is valid in lists */
-					g_set_error (error,
-						     AS_NODE_ERROR,
-						     AS_NODE_ERROR_FAILED,
-						     "Tag %s in %s invalid",
-						     tag_c, tag);
-					return FALSE;
-				}
-			}
-		} else {
-			/* only <p>, <ul> and <ol> is valid here */
-			g_set_error (error,
-				     AS_NODE_ERROR,
-				     AS_NODE_ERROR_FAILED,
-				     "Unknown tag '%s'", tag);
-			return NULL;
-		}
-	}
-
-	/* success */
-	if (str->len > 0)
-		g_string_truncate (str, str->len - 1);
-	return g_strdup (str->str);
-}
-
-/**
- * as_markup_convert_simple:
- * @markup: the text to copy.
- * @error: A #GError or %NULL
- *
- * Converts an XML description into a printable form.
- *
- * Returns: (transfer full): a newly allocated %NULL terminated string
- *
- * Since: 0.1.0
- **/
-gchar *
-as_markup_convert_simple (const gchar *markup, GError **error)
-{
-	return as_markup_convert (markup,
-				  AS_MARKUP_CONVERT_FORMAT_SIMPLE,
-				  error);
-}
+G_DEFINE_QUARK (as-utils-error-quark, as_utils_error)
 
 /**
  * as_hash_lookup_by_locale:
@@ -319,8 +109,9 @@ as_hash_lookup_by_locale (GHashTable *hash, const gchar *locale)
 gboolean
 as_utils_is_stock_icon_name (const gchar *name)
 {
-	_cleanup_bytes_unref_ GBytes *data = NULL;
-	_cleanup_free_ gchar *key = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autofree gchar *key = NULL;
+	gchar *tmp;
 
 	/* load the readonly data section and look for the icon name */
 	data = g_resource_lookup_data (as_get_resource (),
@@ -330,6 +121,11 @@ as_utils_is_stock_icon_name (const gchar *name)
 	if (data == NULL)
 		return FALSE;
 	key = g_strdup_printf ("\n%s\n", name);
+	tmp = g_strstr_len (key, -1, "-symbolic");
+	if (tmp != NULL) {
+		tmp[0] = '\n';
+		tmp[1] = '\0';
+	}
 	return g_strstr_len (g_bytes_get_data (data, NULL), -1, key) != NULL;
 }
 
@@ -346,8 +142,16 @@ as_utils_is_stock_icon_name (const gchar *name)
 gboolean
 as_utils_is_spdx_license_id (const gchar *license_id)
 {
-	_cleanup_bytes_unref_ GBytes *data = NULL;
-	_cleanup_free_ gchar *key = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autofree gchar *key = NULL;
+
+	/* handle invalid */
+	if (license_id == NULL || license_id[0] == '\0')
+		return FALSE;
+
+	/* this is used to map non-SPDX licence-ids to legitimate values */
+	if (g_str_has_prefix (license_id, "LicenseRef-"))
+		return TRUE;
 
 	/* load the readonly data section and look for the icon name */
 	data = g_resource_lookup_data (as_get_resource (),
@@ -373,23 +177,6 @@ as_utils_is_spdx_license_id (const gchar *license_id)
 gboolean
 as_utils_is_blacklisted_id (const gchar *desktop_id)
 {
-	guint i;
-	_cleanup_bytes_unref_ GBytes *data = NULL;
-	_cleanup_free_ gchar *key = NULL;
-	_cleanup_strv_free_ gchar **split = NULL;
-
-	/* load the readonly data section and look for the icon name */
-	data = g_resource_lookup_data (as_get_resource (),
-				       "/org/freedesktop/appstream-glib/as-blacklist-ids.txt",
-				       G_RESOURCE_LOOKUP_FLAGS_NONE,
-				       NULL);
-	if (data == NULL)
-		return FALSE;
-	split = g_strsplit (g_bytes_get_data (data, NULL), "\n", -1);
-	for (i = 0; split[i] != NULL; i++) {
-		if (fnmatch (split[i], desktop_id, 0) == 0)
-			return TRUE;
-	}
 	return FALSE;
 }
 
@@ -406,8 +193,8 @@ as_utils_is_blacklisted_id (const gchar *desktop_id)
 gboolean
 as_utils_is_environment_id (const gchar *environment_id)
 {
-	_cleanup_bytes_unref_ GBytes *data = NULL;
-	_cleanup_free_ gchar *key = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autofree gchar *key = NULL;
 
 	/* load the readonly data section and look for the icon name */
 	data = g_resource_lookup_data (as_get_resource (),
@@ -433,8 +220,8 @@ as_utils_is_environment_id (const gchar *environment_id)
 gboolean
 as_utils_is_category_id (const gchar *category_id)
 {
-	_cleanup_bytes_unref_ GBytes *data = NULL;
-	_cleanup_free_ gchar *key = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autofree gchar *key = NULL;
 
 	/* load the readonly data section and look for the icon name */
 	data = g_resource_lookup_data (as_get_resource (),
@@ -464,7 +251,7 @@ as_utils_spdx_license_tokenize_drop (AsUtilsSpdxHelper *helper)
 {
 	const gchar *tmp = helper->collect->str;
 	guint i;
-	_cleanup_free_ gchar *last_literal = NULL;
+	g_autofree gchar *last_literal = NULL;
 	struct {
 		const gchar	*old;
 		const gchar	*new;
@@ -475,6 +262,7 @@ as_utils_spdx_license_tokenize_drop (AsUtilsSpdxHelper *helper)
 		{ "GFDL",	"GFDL-1.3" },
 		{ "GPL-2",	"GPL-2.0" },
 		{ "GPL-3",	"GPL-3.0" },
+		{ "proprietary", "LicenseRef-proprietary" },
 		{ NULL, NULL } };
 
 	/* nothing from last time */
@@ -539,7 +327,7 @@ as_utils_spdx_license_tokenize_drop (AsUtilsSpdxHelper *helper)
  * with "|". Brackets are added as indervidual tokens and other strings are
  * appended into single tokens where possible.
  *
- * Returns: (transfer full): array of strings
+ * Returns: (transfer full): array of strings, or %NULL for invalid
  *
  * Since: 0.1.5
  **/
@@ -549,6 +337,10 @@ as_utils_spdx_license_tokenize (const gchar *license)
 	guint i;
 	AsUtilsSpdxHelper helper;
 
+	/* handle invalid */
+	if (license == NULL)
+		return NULL;
+
 	helper.last_token_literal = FALSE;
 	helper.collect = g_string_new ("");
 	helper.array = g_ptr_array_new_with_free_func (g_free);
@@ -585,7 +377,7 @@ as_utils_spdx_license_tokenize (const gchar *license)
  *
  * De-tokenizes the SPDX licenses into a string.
  *
- * Returns: (transfer full): string
+ * Returns: (transfer full): string, or %NULL for invalid
  *
  * Since: 0.2.5
  **/
@@ -595,6 +387,10 @@ as_utils_spdx_license_detokenize (gchar **license_tokens)
 	GString *tmp;
 	guint i;
 
+	/* handle invalid */
+	if (license_tokens == NULL)
+		return NULL;
+
 	tmp = g_string_new ("");
 	for (i = 0; license_tokens[i] != NULL; i++) {
 		if (g_strcmp0 (license_tokens[i], "&") == 0) {
@@ -629,9 +425,23 @@ gboolean
 as_utils_is_spdx_license (const gchar *license)
 {
 	guint i;
-	_cleanup_strv_free_ gchar **tokens = NULL;
+	g_auto(GStrv) tokens = NULL;
+
+	/* handle nothing set */
+	if (license == NULL || license[0] == '\0')
+		return FALSE;
+
+	/* no license information whatsoever */
+	if (g_strcmp0 (license, "NONE") == 0)
+		return TRUE;
+
+	/* creator has intentionally provided no information */
+	if (g_strcmp0 (license, "NOASSERTION") == 0)
+		return TRUE;
 
 	tokens = as_utils_spdx_license_tokenize (license);
+	if (tokens == NULL)
+		return FALSE;
 	for (i = 0; tokens[i] != NULL; i++) {
 		if (tokens[i][0] == '@') {
 			if (as_utils_is_spdx_license_id (tokens[i] + 1))
@@ -648,6 +458,111 @@ as_utils_is_spdx_license (const gchar *license)
 	return TRUE;
 }
 
+/**
+ * as_utils_license_to_spdx:
+ * @license: a not-quite SPDX license string, e.g. "GPLv3+"
+ *
+ * Converts a non-SPDX license into an SPDX format string where possible.
+ *
+ * Returns: the best-effort SPDX license string
+ *
+ * Since: 0.5.5
+ **/
+gchar *
+as_utils_license_to_spdx (const gchar *license)
+{
+	GString *str;
+	guint i;
+	guint j;
+	guint license_len;
+	struct {
+		const gchar	*old;
+		const gchar	*new;
+	} convert[] =  {
+		{ " with exceptions",		NULL },
+		{ " with advertising",		NULL },
+		{ " and ",			" AND " },
+		{ " or ",			" OR " },
+		{ "AGPLv3+",			"AGPL-3.0" },
+		{ "AGPLv3",			"AGPL-3.0" },
+		{ "Artistic 2.0",		"Artistic-2.0" },
+		{ "Artistic clarified",		"Artistic-2.0" },
+		{ "Artistic",			"Artistic-1.0" },
+		{ "ASL 1.1",			"Apache-1.1" },
+		{ "ASL 2.0",			"Apache-2.0" },
+		{ "Boost",			"BSL-1.0" },
+		{ "BSD",			"BSD-3-Clause" },
+		{ "CC0",			"CC0-1.0" },
+		{ "CC-BY-SA",			"CC-BY-SA-3.0" },
+		{ "CC-BY",			"CC-BY-3.0" },
+		{ "CDDL",			"CDDL-1.0" },
+		{ "CeCILL-C",			"CECILL-C" },
+		{ "CeCILL",			"CECILL-2.0" },
+		{ "CPAL",			"CPAL-1.0" },
+		{ "CPL",			"CPL-1.0" },
+		{ "EPL",			"EPL-1.0" },
+		{ "Free Art",			"ClArtistic" },
+		{ "GFDL",			"GFDL-1.3" },
+		{ "GPL+",			"GPL-1.0+" },
+		{ "GPLv2+",			"GPL-2.0+" },
+		{ "GPLv2",			"GPL-2.0" },
+		{ "GPLv3+",			"GPL-3.0+" },
+		{ "GPLv3",			"GPL-3.0" },
+		{ "IBM",			"IPL-1.0" },
+		{ "LGPL+",			"LGPL-2.1+" },
+		{ "LGPLv2.1",			"LGPL-2.1" },
+		{ "LGPLv2+",			"LGPL-2.1+" },
+		{ "LGPLv2",			"LGPL-2.1" },
+		{ "LGPLv3+",			"LGPL-3.0+" },
+		{ "LGPLv3",			"LGPL-3.0" },
+		{ "LPPL",			"LPPL-1.3c" },
+		{ "MPLv1.0",			"MPL-1.0" },
+		{ "MPLv1.1",			"MPL-1.1" },
+		{ "MPLv2.0",			"MPL-2.0" },
+		{ "Netscape",			"NPL-1.1" },
+		{ "OFL",			"OFL-1.1" },
+		{ "Python",			"Python-2.0" },
+		{ "QPL",			"QPL-1.0" },
+		{ "SPL",			"SPL-1.0" },
+		{ "zlib",			"Zlib" },
+		{ "ZPLv2.0",			"ZPL-2.0" },
+		{ "Unlicense",			"CC0-1.0" },
+		{ "Public Domain",		"LicenseRef-public-domain" },
+		{ "Copyright only",		"LicenseRef-public-domain" },
+		{ "Proprietary",		"LicenseRef-proprietary" },
+		{ "Commercial",			"LicenseRef-proprietary" },
+		{ NULL, NULL } };
+
+	/* nothing set */
+	if (license == NULL)
+		return NULL;
+
+	/* already in SPDX format */
+	if (as_utils_is_spdx_license (license))
+		return g_strdup (license);
+
+	/* go through the string looking for case-insensitive matches */
+	str = g_string_new ("");
+	license_len = strlen (license);
+	for (i = 0; i < license_len; i++) {
+		gboolean found = FALSE;
+		for (j = 0; convert[j].old != NULL; j++) {
+			guint old_len = strlen (convert[j].old);
+			if (g_ascii_strncasecmp (license + i,
+						 convert[j].old,
+						 old_len) != 0)
+				continue;
+			if (convert[j].new != NULL)
+				g_string_append (str, convert[j].new);
+			i += old_len - 1;
+			found = TRUE;
+		}
+		if (!found)
+			g_string_append_c (str, license[i]);
+	}
+	return g_string_free (str, FALSE);
+}
+
 /**
  * as_pixbuf_blur_private:
  **/
@@ -776,8 +691,8 @@ as_pixbuf_blur (GdkPixbuf *src, gint radius, gint iterations)
 {
 	gint kernel_size;
 	gint i;
-	_cleanup_free_ guchar *div_kernel_size = NULL;
-	_cleanup_object_unref_ GdkPixbuf *tmp = NULL;
+	g_autofree guchar *div_kernel_size = NULL;
+	g_autoptr(GdkPixbuf) tmp = NULL;
 
 	tmp = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (src),
 			      gdk_pixbuf_get_has_alpha (src),
@@ -816,7 +731,7 @@ as_pixbuf_sharpen (GdkPixbuf *src, gint radius, gdouble amount)
 	guchar *p_blurred_row;
 	guchar *p_src;
 	guchar *p_src_row;
-	_cleanup_object_unref_ GdkPixbuf *blurred = NULL;
+	g_autoptr(GdkPixbuf) blurred = NULL;
 
 	blurred = gdk_pixbuf_copy (src);
 	as_pixbuf_blur (blurred, radius, 3);
@@ -886,6 +801,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 					"128x128",
 					"96x96",
 					"256x256",
+					"512x512",
 					"scalable",
 					"48x48",
 					"32x32",
@@ -894,6 +810,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 					NULL };
 	const gchar *sizes_hi_dpi[] = { "128x128",
 					"256x256",
+					"512x512",
 					"scalable",
 					NULL };
 	const gchar *types[] = { "actions",
@@ -910,7 +827,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 				 "status",
 				 "stock",
 				 NULL };
-	_cleanup_free_ gchar *prefix = NULL;
+	g_autofree gchar *prefix = NULL;
 
 	/* fallback */
 	if (destdir == NULL)
@@ -918,7 +835,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 
 	/* is this an absolute path */
 	if (search[0] == '/') {
-		_cleanup_free_ gchar *tmp = NULL;
+		g_autofree gchar *tmp = NULL;
 		tmp = g_build_filename (destdir, search, NULL);
 		if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
 			g_set_error (error,
@@ -935,7 +852,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 	prefix = g_strdup_printf ("%s/usr", destdir);
 	if (!g_file_test (prefix, G_FILE_TEST_EXISTS)) {
 		g_free (prefix);
-		prefix = g_strdup_printf ("%s/files", destdir);
+		prefix = g_strdup (destdir);
 	}
 	if (!g_file_test (prefix, G_FILE_TEST_EXISTS)) {
 		g_set_error (error,
@@ -951,7 +868,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 		for (i = 0; sizes[i] != NULL; i++) {
 			for (m = 0; types[m] != NULL; m++) {
 				for (j = 0; supported_ext[j] != NULL; j++) {
-					_cleanup_free_ gchar *tmp = NULL;
+					g_autofree gchar *tmp = NULL;
 					tmp = g_strdup_printf ("%s/share/icons/"
 							       "%s/%s/%s/%s%s",
 							       prefix,
@@ -970,7 +887,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 	/* pixmap */
 	for (i = 0; pixmap_dirs[i] != NULL; i++) {
 		for (j = 0; supported_ext[j] != NULL; j++) {
-			_cleanup_free_ gchar *tmp = NULL;
+			g_autofree gchar *tmp = NULL;
 			tmp = g_strdup_printf ("%s/share/%s/%s%s",
 					       prefix,
 					       pixmap_dirs[i],
@@ -1043,8 +960,8 @@ as_utils_install_icon (AsUtilsLocation location,
 	int r;
 	struct archive *arch = NULL;
 	struct archive_entry *entry;
-	_cleanup_free_ gchar *data = NULL;
-	_cleanup_free_ gchar *dir = NULL;
+	g_autofree gchar *data = NULL;
+	g_autofree gchar *dir = NULL;
 
 	dir = g_strdup_printf ("%s%s/app-info/icons/%s",
 			       destdir,
@@ -1073,7 +990,7 @@ as_utils_install_icon (AsUtilsLocation location,
 
 	/* decompress each file */
 	for (;;) {
-		_cleanup_free_ gchar *buf = NULL;
+		g_autofree gchar *buf = NULL;
 
 		r = archive_read_next_header (arch, &entry);
 		if (r == ARCHIVE_EOF)
@@ -1100,7 +1017,7 @@ as_utils_install_icon (AsUtilsLocation location,
 		/* update hardlinks */
 		tmp = archive_entry_hardlink (entry);
 		if (tmp != NULL) {
-			_cleanup_free_ gchar *buf_link = NULL;
+			g_autofree gchar *buf_link = NULL;
 			buf_link = g_build_filename (dir, tmp, NULL);
 			archive_entry_update_hardlink_utf8 (entry, buf_link);
 		}
@@ -1108,7 +1025,7 @@ as_utils_install_icon (AsUtilsLocation location,
 		/* update symlinks */
 		tmp = archive_entry_symlink (entry);
 		if (tmp != NULL) {
-			_cleanup_free_ gchar *buf_link = NULL;
+			g_autofree gchar *buf_link = NULL;
 			buf_link = g_build_filename (dir, tmp, NULL);
 			archive_entry_update_symlink_utf8 (entry, buf_link);
 		}
@@ -1143,11 +1060,11 @@ as_utils_install_xml (const gchar *filename,
 		      GError **error)
 {
 	gchar *tmp;
-	_cleanup_free_ gchar *basename = NULL;
-	_cleanup_free_ gchar *path_dest = NULL;
-	_cleanup_free_ gchar *path_parent = NULL;
-	_cleanup_object_unref_ GFile *file_dest = NULL;
-	_cleanup_object_unref_ GFile *file_src = NULL;
+	g_autofree gchar *basename = NULL;
+	g_autofree gchar *path_dest = NULL;
+	g_autofree gchar *path_parent = NULL;
+	g_autoptr(GFile) file_dest = NULL;
+	g_autoptr(GFile) file_src = NULL;
 
 	/* create directory structure */
 	path_parent = g_strdup_printf ("%s%s", destdir, dir);
@@ -1163,7 +1080,7 @@ as_utils_install_xml (const gchar *filename,
 	file_src = g_file_new_for_path (filename);
 	basename = g_path_get_basename (filename);
 	if (origin != NULL) {
-		_cleanup_free_ gchar *basename_new = NULL;
+		g_autofree gchar *basename_new = NULL;
 		tmp = g_strstr_len (basename, -1, ".");
 		if (tmp == NULL) {
 			g_set_error (error,
@@ -1190,7 +1107,7 @@ as_utils_install_xml (const gchar *filename,
 
 	/* fix the origin */
 	if (origin != NULL) {
-		_cleanup_object_unref_ AsStore *store = NULL;
+		g_autoptr(AsStore) store = NULL;
 		store = as_store_new ();
 		if (!as_store_from_file (store, file_dest, NULL, NULL, error))
 			return FALSE;
@@ -1227,8 +1144,8 @@ as_utils_install_filename (AsUtilsLocation location,
 {
 	gboolean ret = FALSE;
 	gchar *tmp;
-	_cleanup_free_ gchar *basename = NULL;
-	_cleanup_free_ gchar *path = NULL;
+	g_autofree gchar *basename = NULL;
+	g_autofree gchar *path = NULL;
 
 	/* default value */
 	if (destdir == NULL)
@@ -1300,11 +1217,19 @@ as_utils_search_token_valid (const gchar *token)
 {
 	guint i;
 	const gchar *blacklist[] = {
-		"and", "the", "desktop", "application", "for", "you", "your",
+		"and", "the", "application", "for", "you", "your",
 		"with", "can", "are", "from", "that", "use", "allows", "also",
 		"this", "other", "all", "using", "has", "some", "like", "them",
 		"well", "not", "using", "not", "but", "set", "its", "into",
 		"such", "was", "they", "where", "want", "only", "about",
+		"uses", "font", "features", "designed", "provides", "which",
+		"many", "used", "org", "fonts", "open", "more", "based",
+		"different", "including", "will", "multiple", "out", "have",
+		"each", "when", "need", "most", "both", "their", "even",
+		"way", "several", "been", "while", "very", "add", "under",
+		"what", "those", "much", "either", "currently", "one",
+		"support", "make", "over", "these", "there", "without", "etc",
+		"main",
 		NULL };
 	if (strlen (token) < 3)
 		return FALSE;
@@ -1341,7 +1266,7 @@ as_utils_search_tokenize (const gchar *search)
 	gchar **values = NULL;
 	guint i;
 	guint idx = 0;
-	_cleanup_strv_free_ gchar **tmp = NULL;
+	g_auto(GStrv) tmp = NULL;
 
 	/* only add keywords that are long enough */
 	tmp = g_strsplit (search, " ", -1);
@@ -1378,8 +1303,10 @@ as_utils_vercmp (const gchar *version_a, const gchar *version_b)
 	gint64 ver_b;
 	guint i;
 	guint longest_split;
-	_cleanup_strv_free_ gchar **split_a = NULL;
-	_cleanup_strv_free_ gchar **split_b = NULL;
+	g_autofree gchar *str_a = NULL;
+	g_autofree gchar *str_b = NULL;
+	g_auto(GStrv) split_a = NULL;
+	g_auto(GStrv) split_b = NULL;
 
 	/* sanity check */
 	if (version_a == NULL || version_b == NULL)
@@ -1390,8 +1317,10 @@ as_utils_vercmp (const gchar *version_a, const gchar *version_b)
 		return 0;
 
 	/* split into sections, and try to parse */
-	split_a = g_strsplit (version_a, ".", -1);
-	split_b = g_strsplit (version_b, ".", -1);
+	str_a = as_utils_version_parse (version_a);
+	str_b = as_utils_version_parse (version_b);
+	split_a = g_strsplit (str_a, ".", -1);
+	split_b = g_strsplit (str_b, ".", -1);
 	longest_split = MAX (g_strv_length (split_a), g_strv_length (split_b));
 	for (i = 0; i < longest_split; i++) {
 
@@ -1444,20 +1373,6 @@ as_ptr_array_find_string (GPtrArray *array, const gchar *value)
 	return NULL;
 }
 
-/**
- * as_utils_guid_is_xdigit:
- **/
-static gboolean
-as_utils_guid_is_xdigit (const gchar *str)
-{
-	guint i;
-	for (i = 0; str[i] != '\0'; i++) {
-		if (!g_ascii_isxdigit (str[i]))
-			return FALSE;
-	}
-	return TRUE;
-}
-
 /**
  * as_utils_guid_is_valid:
  * @guid: string to check
@@ -1471,32 +1386,28 @@ as_utils_guid_is_xdigit (const gchar *str)
 gboolean
 as_utils_guid_is_valid (const gchar *guid)
 {
-	_cleanup_strv_free_ gchar **split = NULL;
+	gint rc;
+	uuid_t uu;
 	if (guid == NULL)
 		return FALSE;
-	split = g_strsplit (guid, "-", -1);
-	if (g_strv_length (split) != 5)
-		return FALSE;
-	if (strlen (split[0]) != 8 || !as_utils_guid_is_xdigit (split[0]))
-		return FALSE;
-	if (strlen (split[1]) != 4 || !as_utils_guid_is_xdigit (split[1]))
-		return FALSE;
-	if (strlen (split[2]) != 4 || !as_utils_guid_is_xdigit (split[2]))
-		return FALSE;
-	if (strlen (split[3]) != 4 || !as_utils_guid_is_xdigit (split[3]))
-		return FALSE;
-	if (strlen (split[4]) != 12 || !as_utils_guid_is_xdigit (split[4]))
-		return FALSE;
-	return TRUE;
+	rc = uuid_parse (guid, uu);
+	return rc == 0;
 }
 
 /**
  * as_utils_guid_from_string:
  * @str: A source string to use as a key
  *
- * Returns a GUID for a given string. This uses SHA1 and some string
- * modification so even small differences in the @str will produce radically
- * different GUID return values.
+ * Returns a GUID for a given string. This uses a hash and so even small
+ * differences in the @str will produce radically different return values.
+ *
+ * The implementation is taken from RFC4122, Section 4.1.3; specifically
+ * using a type-5 SHA-1 hash with a DNS namespace.
+ * The same result can be obtained with this simple python program:
+ *
+ *    #!/usr/bin/python
+ *    import uuid
+ *    print uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
  *
  * Returns: A new GUID, or %NULL if the string was invalid
  *
@@ -1505,13 +1416,203 @@ as_utils_guid_is_valid (const gchar *guid)
 gchar *
 as_utils_guid_from_string (const gchar *str)
 {
+	const gchar *namespace_id = "6ba7b810-9dad-11d1-80b4-00c04fd430c8";
+	gchar guid_new[37]; /* 36 plus NUL */
+	gsize digestlen = 20;
+	guint8 hash[20];
+	gint rc;
+	uuid_t uu_namespace;
+	uuid_t uu_new;
+	g_autoptr(GChecksum) csum = NULL;
+
+	/* invalid */
+	if (str == NULL)
+		return NULL;
+
+	/* convert the namespace to binary */
+	rc = uuid_parse (namespace_id, uu_namespace);
+	g_assert (rc == 0);
+
+	/* hash the namespace and then the string */
+	csum = g_checksum_new (G_CHECKSUM_SHA1);
+	g_checksum_update (csum, (guchar *) uu_namespace, 16);
+	g_checksum_update (csum, (guchar *) str, strlen(str));
+	g_checksum_get_digest (csum, hash, &digestlen);
+
+	/* copy most parts of the hash 1:1 */
+	memcpy(uu_new, hash, 16);
+
+	/* set specific bits according to Section 4.1.3 */
+	uu_new[6] = (guint8) ((uu_new[6] & 0x0f) | (5 << 4));
+	uu_new[8] = (guint8) ((uu_new[8] & 0x3f) | 0x80);
+
+	/* return as a string */
+	uuid_unparse (uu_new, guid_new);
+	return g_strdup (guid_new);
+}
+
+/**
+ * as_utils_version_from_uint32:
+ * @val: A uint32le version number
+ * @flags: flags used for formatting, e.g. %AS_VERSION_PARSE_FLAG_USE_TRIPLET
+ *
+ * Returns a dotted decimal version string from a 32 bit number.
+ *
+ * Returns: A version number, e.g. "1.0.3"
+ *
+ * Since: 0.5.2
+ **/
+gchar *
+as_utils_version_from_uint32 (guint32 val, AsVersionParseFlag flags)
+{
+	if (flags & AS_VERSION_PARSE_FLAG_USE_TRIPLET) {
+		return g_strdup_printf ("%i.%i.%i",
+					(val >> 24) & 0xff,
+					(val >> 16) & 0xff,
+					val & 0xffff);
+	}
+	return g_strdup_printf ("%i.%i.%i.%i",
+				(val >> 24) & 0xff,
+				(val >> 16) & 0xff,
+				(val >> 8) & 0xff,
+				val & 0xff);
+}
+
+/**
+ * as_utils_version_from_uint16:
+ * @val: A uint16le version number
+ * @flags: flags used for formatting, e.g. %AS_VERSION_PARSE_FLAG_USE_TRIPLET
+ *
+ * Returns a dotted decimal version string from a 16 bit number.
+ *
+ * Returns: A version number, e.g. "1.3"
+ *
+ * Since: 0.5.2
+ **/
+gchar *
+as_utils_version_from_uint16 (guint16 val, AsVersionParseFlag flags)
+{
+	return g_strdup_printf ("%i.%i",
+				(val >> 8) & 0xff,
+				val & 0xff);
+}
+
+/**
+ * as_utils_version_parse:
+ * @version: A version number
+ *
+ * Returns a dotted decimal version string from a version string. The supported
+ * formats are:
+ *
+ * - Dotted decimal, e.g. "1.2.3"
+ * - Base 16, a hex number *with* a 0x prefix, e.g. "0x10203"
+ * - Base 10, a string containing just [0-9], e.g. "66051"
+ * - Date in YYYYMMDD format, e.g. 20150915
+ *
+ * Anything with a '.' or that doesn't match [0-9] or 0x[a-f,0-9] is considered
+ * a string and returned without modification.
+ *
+ * Returns: A version number, e.g. "1.0.3"
+ *
+ * Since: 0.5.2
+ */
+gchar *
+as_utils_version_parse (const gchar *version)
+{
+	gchar *endptr = NULL;
+	guint64 tmp;
+	guint base;
+	guint i;
+
+	/* already dotted decimal */
+	if (g_strstr_len (version, -1, ".") != NULL)
+		return g_strdup (version);
+
+	/* is a date */
+	if (g_str_has_prefix (version, "20") &&
+	    strlen (version) == 8)
+		return g_strdup (version);
+
+	/* convert 0x prefixed strings to dotted decimal */
+	if (g_str_has_prefix (version, "0x")) {
+		version += 2;
+		base = 16;
+	} else {
+		/* for non-numeric content, just return the string */
+		for (i = 0; version[i] != '\0'; i++) {
+			if (!g_ascii_isdigit (version[i]))
+				return g_strdup (version);
+		}
+		base = 10;
+	}
+
+	/* convert */
+	tmp = g_ascii_strtoull (version, &endptr, base);
+	if (endptr != NULL && endptr[0] != '\0')
+		return g_strdup (version);
+	if (tmp == 0 || tmp < 0xff)
+		return g_strdup (version);
+	return as_utils_version_from_uint32 (tmp, AS_VERSION_PARSE_FLAG_USE_TRIPLET);
+}
+
+/**
+ * as_utils_string_replace:
+ * @string: The #GString to operate on
+ * @search: The text to search for
+ * @replace: The text to use for substitutions
+ *
+ * Performs multiple search and replace operations on the given string.
+ *
+ * Returns: the number of replacements done, or 0 if @search is not found.
+ *
+ * Since: 0.5.11
+ **/
+guint
+as_utils_string_replace (GString *string, const gchar *search, const gchar *replace)
+{
 	gchar *tmp;
-	tmp = g_compute_checksum_for_string (G_CHECKSUM_SHA1, str, -1);
-	tmp[8] = '-';
-	tmp[13] = '-';
-	tmp[18] = '-';
-	tmp[23] = '-';
-	tmp[36] = '\0';
-	g_assert (as_utils_guid_is_valid (tmp));
-	return tmp;
+	guint count = 0;
+	guint search_idx = 0;
+	guint replace_len;
+	guint search_len;
+
+	g_return_val_if_fail (string != NULL, 0);
+	g_return_val_if_fail (search != NULL, 0);
+	g_return_val_if_fail (replace != NULL, 0);
+
+	/* nothing to do */
+	if (string->len == 0)
+		return 0;
+
+	search_len = strlen (search);
+	replace_len = strlen (replace);
+
+	do {
+		tmp = g_strstr_len (string->str + search_idx, -1, search);
+		if (tmp == NULL)
+			break;
+
+		/* advance the counter in case @replace contains @search */
+		search_idx = tmp - string->str;
+
+		/* reallocate the string if required */
+		if (search_len > replace_len) {
+			g_string_erase (string, search_idx,
+					search_len - replace_len);
+			memcpy (tmp, replace, replace_len);
+		} else if (search_len < replace_len) {
+			g_string_insert_len (string, search_idx, search,
+					     replace_len - search_len);
+			/* we have to treat this specially as it could have
+			 * been reallocated when the insertion happened */
+			memcpy (string->str + search_idx, replace, replace_len);
+		} else {
+			/* just memcmp in the new string */
+			memcpy (tmp, replace, replace_len);
+		}
+		search_idx += replace_len;
+		count++;
+	} while (TRUE);
+
+	return count;
 }
